// Objet application
function Application(racine) {
	// Recherche d'une nouvelle version
	if(window.applicationCache) {
		window.applicationCache.addEventListener('updateready',function() {
			// on ne demande la mise à jour
			// que si le joueur n'est pas entrain de jouer
			if((!window['VueMono'])||this.vue!=window['VueMono'])
				this.montrerVue('Maj');
		}.bind(this));
	}
	// stockage de l'élément HTML racine
	this.racine=racine;
	// création d'un gestionnaire de commande
	this.gestionnaire=new GestionnaireCommandes(racine);
	// création d'un gestionnaire de sons et ajout des sons
	this.sons=new GestionnaireSons('sounds',
		// joue le son de fond une fois chargé
		function() { this.sons.jouer('mountainman'); }.bind(this));
	this.sons.enregistrer('mountainman',['mid','mp3'],Infinity,0.2);
	this.sons.enregistrer('carte1',['ogg']);
	this.sons.enregistrer('carte2',['ogg']);
	this.sons.enregistrer('bien',['ogg','mp3']);
	this.sons.enregistrer('pasbien',['ogg']);
	this.sons.enregistrer('applaudissements',['ogg']);
	this.sons.enregistrer('alerte1',['ogg']);
	this.sons.enregistrer('alerte2',['ogg']);
	this.sons.enregistrer('perdu',['ogg']);
	this.sons.enregistrer('gagne',['ogg']);
	// Récupération de la préférence son coupé
	try {
		if(window.localStorage&&window.localStorage.sonCoupé)
			this.sons.muter(!!window.localStorage.sonCoupé);
	} catch(e) {}
	// Enregistrement de la commande de changement de vue
	this.gestionnaire.ajouterCommande('changerVue',
		this.changerVue.bind(this));
	// stockage de l'élément pour afficher des messages
	this.message=document.querySelector('div.app div.message p');
	// on crée la variable contenant les essais websocket
	this.essaisWS=0;
	// on affiche la première vue
	this.montrerVue('Accueil');
}

// Gestion des messages
Application.prototype.afficherMessage=function (texte,durée,callback) {
	if(this.messageTimeout)
		clearTimeout(this.messageTimeout);
	this.messageTimeout=setTimeout(this.cacherMessage.bind(this),durée||1500);
	this.messageCallback=callback;
	this.message.firstChild.textContent=texte;
	this.message.parentNode.classList.add('affiche');
};

Application.prototype.cacherMessage=function () {
	this.message.parentNode.classList.remove('affiche');
	this.messageTimeout=null;
	this.messageCallback&&this.messageCallback();
	this.messageCallback=null;
};

// Gestion des vues
Application.prototype.changerVue=function (event,params) {
	this.montrerVue(params.vue);
};

Application.prototype.montrerVue=function (name) {
	// on désamorce la vue précédente
	if(this.vueAffichée)
		this.vueAffichée.uninit();
	// on lance la vue suivante
	// soit la vue a un constructeur dédié
	// soit on utilise le constructeur générique Vue
	if(undefined===window['Vue'+name]) {
		var requete=new XMLHttpRequest();
		requete.open('GET','javascript/Vue'+name+'.js',false);
		requete.send();
		if(200!==requete.status)
			window['Vue'+name]=null;
		else
			eval.bind(window)(requete.responseText);
	}
	this.vueAffichée=new (window['Vue'+name]||window['Vue'])();
	this.vueAffichée.init(this,name);
};

// Gestion des websockets
Application.prototype.connexion=function () {
	// on garde une référence au callback de message
	if(this.connexionWS&&this.connexionWS.onmessage)
		var retourMessage=this.connexionWS.onmessage;
	// si on a 3 essais de connexion infructueux
	if(this.essaisWS>2) {
		// on annule tout
		this.connexionWS.close();
		this.connexionWS=null;
		this.essaisWS=0;
		// on affiche la vue erreur réseau
		this.montrerVue('Reseau');
	}
	// sinon, on retente une connexion
	this.essaisWS++;
	this.connexionWS = new WebSocket('ws://'+document.location.hostname+':8124');
	// on crée un timer pour abandonner la connexion
	// au bout de 5 secondes
	this.timeout=setTimeout(function() {
		if(this.essaisWS>0) {
			this.connexionWS.close();
			this.connexion();
		}
	}.bind(this),5000);
	// à l'ouverture de la connexion, on envoi les infos
	this.connexionWS.onopen = function () {
		if(retourMessage)
			this.connexionWS.onmessage=retourMessage;
		// on supprime le timeout
		if(this.timeout)
			clearTimeout(this.timeout);
		// on remet à zéro les essais
		this.essaisWS=0;
		// on envoi les infos utilisateur
		this.identification();
	}.bind(this);
	// on logge les erreurs pour débogage
	this.connexionWS.onerror = function (error) {
		console.log(error);
	}.bind(this);
	// on écoute toute fermeture
	this.connexionWS.onclose = function () {
		// on supprime le timeout
		if(this.timeout)
			clearTimeout(this.timeout);
		// On tente une reconnexion
		setTimeout(this.connexion.bind(this),2000);
	}.bind(this);
	// on écoute toute fermeture
	this.connexionWS.addEventListener('message', function (event) {
		console.log(event.data);
	});
};

// méthode d'envoi du profil
Application.prototype.identification=function () {
	this.connexionWS.send(JSON.stringify({
		'type':'connexion',
		'nom':this.utilisateur.nom,
		'sexe':this.utilisateur.sexe,
		'image1':this.utilisateur.image1,
		'image2':this.utilisateur.image2,
		'sessid':(this.utilisateur.sessid||'')
	}));
};

// méthode de déconnexion
Application.prototype.déconnexion=function () {
	// on supprime l'écouteur d'évènement
	this.connexionWS.onclose=null;
	// on ferme la connexion
	this.connexionWS.close();
	// on supprime toute référence à la connexion
	this.connexionWS=null;
};

// On lance l'application
var app=new Application(document.querySelector('div.app'));
