// AMD + global
(function(root,define){ define(['Vue'], function (Vue) {

	// Vue pour jeu en solo
	function VueMono() { }

	// Hérite de Vue
	VueMono.prototype=new Vue();

	// Initialisation
	VueMono.prototype.init=function (app,name) {
		// Appel de la fonction parente
		Vue.prototype.init.bind(this)(app,name);
		// Enregistrement des commandes de la vue
		this.commande('retourner');
		this.commande('recommencer');
		// Acquisition d'une référence aux éléments d'affichage
		this.niveau=document.querySelector('div.vue.choisie ul.infos '
			+'li:first-child span').firstChild;
		this.temps=document.querySelector('div.vue.choisie ul.infos '
			+'li:nth-child(2) span').firstChild;
		this.essais=document.querySelector('div.vue.choisie ul.infos '
			+'li:nth-child(3) span').firstChild;
		// Chargement de la liste des univers de jeu
		var requete = new XMLHttpRequest();
		requete.open('GET','/univers/liste.json',false);
		requete.send(null);
		if(requete.status!=200)
			throw Error('Impossible de récupérer la liste des univers !');
		this.listeUnivers=JSON.parse(requete.responseText);
		// Mise en route
		this.recommencer();
	}

	// Recommence le jeu au début
	VueMono.prototype.recommencer=function () {
		// Mise à zéro du niveau courant
		this.niveau.textContent=1;
		// distribution des cartes
		this.distribuer();
	}

	// Commence un niveau
	VueMono.prototype.distribuer=function () {
		// Remise à 0 des essais
		this.essais.textContent=0;
		// Remise à 0 des paires trouvées
		this.paireTrouvées=0;
		// Récupération des cartes
		var pileCartes=Array.prototype.slice.call(
			document.querySelectorAll('div.vue.choisie div.cartes p'),0);
		// Chargement d'un fichier univers au hasard
		var requete = new XMLHttpRequest();
		var nomUnivers=this.listeUnivers[
			Math.floor(Math.random()*this.listeUnivers.length)];
		requete.open('GET','univers/'+nomUnivers+'.json',false);
		requete.send();
		if(requete.status!=200)
			throw Error('Impossible de récupérer l\'univers !');
		var univers=JSON.parse(requete.responseText);
		// Distribution des cartes
		this.cartes=[];
		// On itère pour chaque paire de cartes
		for(var i=(pileCartes.length/2)-1; i>=0; i--)	{
			// on retire deux cartes de la pile, au hasard
			var carte1=pileCartes.splice(Math.floor(Math.random()*pileCartes.length),1)[0];
			var carte2=pileCartes.splice(Math.floor(Math.random()*pileCartes.length),1)[0];
			// on sélectionne un type de carte dans l'univers actuel
			// on supprime ce type de l'univers pour éviter de le réutiliser
			carte1.typeCarte=carte2.typeCarte=univers.cartes.pop();
			// Si l'univers déclare un jeu d'image
			if(univers.image) {
				// on ajoute l'image en fond de la carte
				// et on modifie sa position sur l'axe des x
				// pour que la bonne image apparaisse
				carte1.firstChild.firstChild.style.backgroundImage='url(/images/'+univers.image+')';
				carte1.firstChild.firstChild.style.backgroundPosition='-'+(i*univers.largeurTuile)+'px center';
				carte2.firstChild.firstChild.style.backgroundImage='url(/images/'+univers.image+')';
				carte2.firstChild.firstChild.style.backgroundPosition='-'+(i*univers.largeurTuile)+'px center';
			}
			// sinon, on supprime les image de fond éventuelles
			else {
				carte1.firstChild.firstChild.style.backgroundImage='none';
				carte2.firstChild.firstChild.style.backgroundImage='none';
			}
			// on ajoute le texte du type de carte à la carte
			carte1.firstChild.firstChild.firstChild.firstChild.textContent=carte1.typeCarte.texte;
			carte2.firstChild.firstChild.firstChild.firstChild.textContent=carte2.typeCarte.texte;
			// on insère les cartes dans le jeu
			this.cartes.push(carte2.jumelle=carte1,carte1.jumelle=carte2);
		}
		// Remise à zéro des classes
		this.raz();
		// Remise à 0 des sons d'alerte
		this.alert1Sonnée=false;
		this.alert2Sonnée=false;
		// Remise à zéro du timer
		this.tempsMax=new Date(Date.now()+5000+(120000/parseInt(this.niveau.textContent)));
		// Mise en route du compte à rebours
		if(univers.image) {
			// on crée un nœud de type image
			var img=document.createElement('img');
			img.setAttribute('src','images/'+univers.image);
			// on démarre quand elle est chargée
			img.onload=this.majTemps.bind(this);
		}
		else
			this.majTemps();
	}

	// mise à jour du compte à rebours
	VueMono.prototype.majTemps=function () {
		// on supprime tout timeout préalable
		clearTimeout(this.timeoutTemps);
		// Différence entre tempsMax et moment actuel en secondes
		var tempsRestant=(this.tempsMax.getTime()-Date.now())/1000;
		// Si le temps restant est négatif, on le remet à 0
		if(0>=tempsRestant)
			tempsRestant=0;
		// Affichage dans l'élément adequat
		this.temps.textContent=Math.floor(tempsRestant/60)
			+'\''+(Math.floor(tempsRestant%60)<10?'0':'')+Math.floor(tempsRestant%60);
		// diffusion des alertes
		if(tempsRestant<20&&!this.alert1Sonnée) {
			this.app.sons.play('alerte1');
			this.temps.parentNode.classList.add('warn1');
			this.alert1Sonnée=true;
		}
		if(tempsRestant<10&&!this.alert2Sonnée) {
			this.app.sons.play('alerte2');
			this.temps.parentNode.classList.add('warn2');
			this.alert2Sonnée=true;
		}
		// Programmation du prochain affichage
		if(tempsRestant>0)
			this.timeoutTemps=setTimeout(this.majTemps.bind(this), 1000);
		// Plus de temps, le joueur a perdu
		else {
			this.timeoutTemps=null;
			this.app.sons.play('perdu');
			this.app.afficherMessage('Perdu !',3000,this.recommencer.bind(this));
		}
	}

	// Commande pour retourner une carte
	VueMono.prototype.retourner=function (event, params) {
		// on vérifie qu'on est pas entrain d'afficher un résultat
		if(this.timeoutResultat)
			return false;
		// récupération de la carte cliquée
		var carte=document.querySelector(
			'div.vue.choisie div.cartes p:nth-child('+params['carte']+')');
		// on vérifie qu'elle n'est pas déjà trouvée ou sélectionnée
		if(carte.classList.contains('trouvee')
			||carte.classList.contains('choisie'))
			return false;
		// on la sélectionne
		carte.classList.add('face');
		carte.classList.add('choisie');
		// ajout de la couleur associée à la carte
		carte.firstChild.style.backgroundColor='#'+carte.typeCarte.couleur;
		// Si aucune carte choisie on garde une référence à celle-ci
		if(!this.carte1) {
			this.carte1=carte;
			this.app.sons.play('carte1');
		}
		// sinon, on vérifie si elle sont jumelles
		else {
			this.app.sons.play('carte2');
			this.carte2=carte;
			// si elles sont jumelle on appelle la méthode gagner
			// sinon on appelle la méthode perdre
			this.timeoutResultat=setTimeout(
				this[(this.carte1.jumelle==this.carte2?'gagner':'perdre')].bind(this),
				(this.carte1.jumelle==this.carte2?100:700));
		}
	}

	// action si la paire est pas bonne
	VueMono.prototype.perdre=function () {
		this.app.sons.play('pasbien');
		this.timeoutResultat=null;
		this.carte1.classList.toggle('face');
		this.carte1.classList.toggle('choisie');
		this.carte2.classList.toggle('face');
		this.carte2.classList.toggle('choisie');
		this.carte1=null;
		this.carte2=null;
		// on ajoute un essai infructueux
		this.essais.textContent=parseInt(this.essais.textContent,10)+1;
	}

	// action si la paire est bonne
	VueMono.prototype.gagner=function () {
		this.app.sons.play('bien');
		// on passe les deux carte en état trouvé
		this.timeoutResultat=null;
		this.carte1.classList.toggle('choisie');
		this.carte1.classList.toggle('trouvee');
		this.carte2.classList.toggle('choisie');
		this.carte2.classList.toggle('trouvee');
		// on réinitialise les cartes sélectionnées
		this.carte1=null;
		this.carte2=null;
		// on ajoute la paire trouvéee
		this.paireTrouvées++;
		// si toutes les paires sont trouvée
		if(this.paireTrouvées==this.cartes.length/2) {
			this.niveau.textContent=parseInt(this.niveau.textContent,10)+1;
			clearTimeout(this.timeoutTemps);
			this.timeoutTemps=null;
			// jouer le son d'applaudissements
			this.app.sons.play('applaudissements');
			// on félicite le joueur et on programme une nouvelle distribution
			this.app.afficherMessage('Bravo !',1500,this.distribuer.bind(this));
		}
	}

	// on remet la vue dans son état initial
	VueMono.prototype.raz=function () {
		this.carte1=null;
		this.carte2=null;
		if(this.timeoutResultat) {
			clearTimeout(this.timeoutResultat);
			this.timeoutResultat=null;
		}
		this.temps.parentNode.classList.remove('warn1');
		this.temps.parentNode.classList.remove('warn2');
		for(var i=this.cartes.length-1; i>=0; i--) {
			this.cartes[i].classList.remove('choisie');
			this.cartes[i].classList.remove('trouvee');
			this.cartes[i].classList.remove('face');
		}
	}

	// désinitialisation de la vue
	VueMono.prototype.uninit=function (app) {
		// suppression de tout callback de message
		this.app.messageCallback=null;
		// suppression du compte à rebours
		clearTimeout(this.timeoutTemps);
		// remise à zéro de la vue
		this.raz();
		// appel de la fonction parente
		Vue.prototype.uninit.bind(this)();
	}

	return VueMono;

})})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});
