// Objet de gestion des sons
function GestionnaireSons(dossier,fonctionRetour) {
	// indique si le son est coupé
	this.coupé=false;
	// contient les objets son chargés
	this.sons={};
	// contient les objets son à charger
	this.sonsÀCharger=new Array();
	// fonction appelée quand les sons sont chargés
	this.sonsChargés=fonctionRetour;
	// extension à utiliser
	var son=document.createElement('audio');
	this.exts=[];
	if(son.canPlayType('audio/ogg'))
		this.exts.push('ogg');
	if(son.canPlayType('audio/mp3'))
		this.exts.push('mp3');
	if(son.canPlayType('audio/x-midi'))
		this.exts.push('mid');
	// indique le dossier contenant les sons
	this.dossier=dossier;
}

// enregistre un son à charger
GestionnaireSons.prototype.enregistrer = function(nom, extensions, itérations, volume) {
	// création de l'objet Audio
	var son=new Audio();
	// Ajout dans la liste des sons à charger
	this.sonsÀCharger.push(son);
	son.setAttribute('preload','auto');
	if(extensions.every(function(ext) {
		if(-1===this.exts.indexOf(ext))
			return true;
		son.setAttribute('src',this.dossier+'/'+nom+'.'+ext);
		return false;
	}.bind(this)))
		return;
	// gestion du nombre d'itérations
	if(Infinity===itérations)
		son.setAttribute('loop','loop');
	else if(itérations>1)
		son.setAttribute('data-iterations',itérations);
	if(volume)
		son.setAttribute('data-volume',volume);
	son.setAttribute('data-nom',nom);
	// Ajout d'un écouteur d'évènement pour savoir
	// quand le son sera chargé
	son.addEventListener('canplaythrough', this.captureChargement.bind(this));
},

// dépile un son du tableau des sons à charger
GestionnaireSons.prototype.captureChargement = function(event) {
	// on récupère l'index du son chargé
	var index=this.sonsÀCharger.indexOf(event.target);
	if(index>=0) {
		var son=this.sonsÀCharger.splice(index,1)[0];
		this.sons[son.getAttribute('data-nom')]=son;
	}
	// si il n'y a plus de sons à charger
	// on appelle la fonction de retour
	if(this.sonsChargés&&!this.sonsÀCharger.length)
		this.sonsChargés();
};

// Joue un son
GestionnaireSons.prototype.jouer = function(nom,itérations) {
	// si le son existe
	if(this.sons[nom]&&(this.sons[nom].hasAttribute('loop')||!this.coupé)) {
		// récupération du nombre d'itérations
		if(!itérations) {
			itérations=1;
			if(this.sons[nom].hasAttribute('data-iterations'))
				itérations=parseInt(
					this.sons[nom].getAttribute('data-iterations'),10);
		}
		// on clone le nœuds e on joue le son
		this.sons[nom].actuellementJoué=
			this.sons[nom].cloneNode();
		if(this.sons[nom].hasAttribute('data-volume'))	{
			this.sons[nom].actuellementJoué.volume=
				this.sons[nom].actuellementJoué
					.getAttribute('data-volume');
		}
		 // si le son n'est pas coupé ou si le son est un fond sonore
		if(!this.coupé)	{
			this.sons[nom].actuellementJoué.play();
			this.sons[nom].actuellementJoué
				.addEventListener('ended', function()	{
				if(--itérations)
					this.sons[nom].actuellementJoué.play();
				else
					this.sons[nom].actuellementJoué=null;
			}.bind(this));
		}
	}
};

// Arrête un son
GestionnaireSons.prototype.arrêter = function(nom) {
	if(this.sons[nom].actuellementJoué)
		this.sons[nom].actuellementJoué.pause();
	this.sons[nom].actuellementJoué=null;
};

// Change le volume d'un son
GestionnaireSons.prototype.muter = function(couper) {
		for(var nom in this.sons) {
			if(this.sons[nom].actuellementJoué) {
				this.sons[nom].actuellementJoué[couper?'pause':'play']();
			}
		}
	this.coupé=couper;
};
