// Modules
var http=require('http'),
	fs=require('fs'),
	url = require('url'),
	ws = require('websocket').server;
// Constantes
const MIME_TYPES={
	'html':'text/html',
	'js':'text/javascript',
	'manifest':'text/cache-manifest',
	'css': 'text/css',
	'png': 'image/png',
	'jpg': 'image/jpeg',
	'ico': 'image/ico',
	'mp3': 'audio/mp3',
	'ogg': 'audio/ogg',
	'mid': 'audio/x-midi',
	'json': 'application/json',
	'webapp':'application/x-web-app-manifest+json'
};

// Variables
var racine=__dirname+'/www', // on ajoute la racine par défaut
	domaine='127.0.0.1',
	port=8124;

// on essaie de récupérer l'argument
if(process.argv[2])
	racine=process.argv[2];
// on vérifie si la racine du serveur
// est bien un dossier
if(!fs.statSync(racine).isDirectory())
	throw Error('La racine du serveur est mauvaise.');

var serveurHTTP=http.createServer(function (request, response) {
	// on parse l'url
	var urlParsée=url.parse(request.url);
	// on ne fait rien si c'est une connexion websocket
	if(urlParsée.pathname==='/ws')
		return;
	// contenu dynamique
	// liste des univers
	if(urlParsée.pathname==='/univers/liste.json'&&
		(request.method=='HEAD'||request.method=='GET')) {
		// On liste le contenu du dossier /univers
		fs.readdir(racine+'/univers',function(erreur,fichiers) {
			var index=-1;
			// en cas d'erreur, on stoppe tout
			if(erreur) {
				response.writeHead(500);
				response.end();
				throw Error('Impossible de lire le dossier des univers.');
			}
			// sinon, on envoie un code de succès
			response.writeHead(200,{'Content-Type':MIME_TYPES['json']});
			// on supprime les extensions de fichier
			fichiers.map(function(fichier,i) {
				fichiers[i]=fichier.replace(/.json$/,'');
				// on garde l'index du fichier liste éventuellement présent
				if('liste'==fichiers[i])
					index=i;
			});
			// si le fichier liste existe, on le supprime
			if(-1!==index)
				fichiers.splice(index,1);
			// on envoi le contenu en json
			response.end(JSON.stringify(fichiers));
		});
		// on schinte le serveur statique
		return;
	}
	// génération du manifeste
	if(urlParsée.pathname==='/application.manifest'&&
		(request.method=='HEAD'||request.method=='GET')) {
		// Parallélisation des listing du contenu de dossiers
		var dossiers=['univers','javascript','images','sounds','css'];
		var listings=[];
		var dossiersRestants=dossiers.length;
		dossiers.forEach(function(nom) {
			fs.readdir(racine+'/'+nom,function(erreur,fichiers) {
				// en cas d'erreur, on stoppe tout
				if(erreur) {
					response.writeHead(500);
					response.end();
					throw Error('Impossible de lire le dossier "'+nom+'".');
				}
				// on garde une référence sur la liste de fichiers
				listings[nom]=fichiers;
				// si tous les dossiers ont été lus
				if(0==--dossiersRestants) {
					// sinon, on envoie un code de succès
					response.writeHead(200,{'Content-Type':MIME_TYPES['manifest']});
					// puis on génère le manifeste
					response.write('CACHE MANIFEST\n# v 1.0:'+process.pid+'\n\nCACHE:\n/index.html\n');
					// on itère sur chaque listing
					dossiers.forEach(function(nom) {
						// puis sur chaque fichier du listing
						for(var i=listings[nom].length-1; i>=0; i--) {
							// enfin on écrit la ligne du manifeste (sauf pour liste.json)
							// qui est dans la section fallback
							if('liste.json'!==listings[nom][i])
								response.write('/'+nom+'/'+listings[nom][i]+'\n');
						}
					});
					// on finalise le manifeste
					response.end('\nFALLBACK:\n/univers/liste.json /univers/liste.json\n\nNETWORK:\n*\n');
				}
			});
		});
		// on schinte le serveur statique
		return;
	}
	// contenu statique
	// accès en lecture uniquement
	if(request.method!='HEAD'&&request.method!='GET') {
		response.writeHead(401);
		response.end();
		return;
	}
	// vérification de l'existence
	// on refuse les paramètres d'URL
	if('search' in urlParsée) {
		response.writeHead(401);
		response.end();
	}
	// on redirige vers l'index si le client
	// atteint la racine du site
	if('/'===urlParsée.pathname||!urlParsée.pathname) {
		response.writeHead(301,{'Location':'/index.html'});
		response.end();
		return;
	}
	// On vérifie l'existence du fichier correspondant
	fs.stat(racine+urlParsée.pathname,
		function(erreur,resultat) {
			var headers={}, code=0, début=0, fin;
			// si une erreur survient
			// on envoi une erreur 404
			if(erreur||!resultat.isFile()) {
				response.writeHead(404);
				response.end();
				return;
			}
			// on trouve l'extension
			// si elle n'est pas supportée
			// on envoie une erreur 500
			// on stoppe le serveur
			var ext=urlParsée.pathname
				.replace(/^(?:.*)\.([a-z0-9]+)$/,'$1');
			if(!MIME_TYPES[ext]) {
				response.writeHead(500);
				response.end();
				throw Error('Fichier au type mime non-supporté');
			}
			headers['Content-Type']=MIME_TYPES[ext];
			headers['Content-Length']=resultat.size;
			// on vérifie si le contenu doit être morcelé
			if(request.headers.range) {
				// on supprime la chaîne bytes=
				// de l'entête Range et on récupère
				// un tableau contenant le début et la fin
				// de la plage d'octets
				var morceaux = request.headers.range.replace(/bytes=/, "").split("-");
				// on convertit la variable début en entier
				début = parseInt(morceaux[0],10);
				// si il y a une fin, on convertit en entier
				// sinon, la fin est la taille du fichier
				fin =  morceaux[1] ? parseInt(morceaux[1], 10) :
					headers['Content-Length']-1; 
				headers['Content-Range'] = 'bytes ' + début + '-' + fin + '/'
					+ (headers['Content-Length']);
				headers['Accept-Ranges'] = 'bytes';
				headers['Content-Length']= (fin-début)+1;
				headers['Transfer-Encoding'] = 'chunked';
				headers['Connection'] = 'close';
				code=206;
			}
		else
			code=200;

		// on envoi le code et les entêtes
		response.writeHead(code, headers);
		// si la méthode est GET alors on envoi
		// aussi le contenu du fichier
		if(request.method=='GET')
			{
			// on transvase le flux de lecture
			// en prenant soin d'indiquer la
			// plage d'octets concernés 
			fs.createReadStream(racine
			+urlParsée.pathname,{start: début, end: fin})
			.pipe(response);
			}
		else
			response.end();
	}); 
}).listen(port);

console.log('Serveur démarré sur http://'+domaine+':'+port+'/');
console.log('Racine du serveur : '+racine);



// on crée le serveur websocket en lui passant
// une référence au serveur HTTP
var serveurWebsocket = new ws({
	httpServer: serveurHTTP,
	autoAcceptConnections: false
});

// On écoute les demandes de connexion
serveurWebsocket.on('request', function(request) {

	// Si l'origine de la connexion n'est pas notre site, on la rejette
	if(-1===request.origin.indexOf('http://'+domaine+':'+port)) {
		console.log(new Date()+': Origine de la connexion rejetée ('+request.origin+').');
		request.reject();
		return;
	}

  var connection = request.accept(null, request.origin);
  console.log((new Date()) + ': Nouvelle connexion acceptée.');
	// on écoute les messages reçus depuis cette connexion
  connection.on('message', function(message) {
		// on ne traite que les message encodés en utf8
    if ('utf8' === message.type) {
	    console.log('Message reçu: ' + message.utf8Data);
      connection.sendUTF(message.utf8Data);
		}
	});
	
	// on écoute la fermeture des connexions
  connection.on('close', function(reasonCode, description) {
	  console.log((new Date()) + ': Connection fermée (' + connection.remoteAddress + ').');
	});

});
