// Modules
var http=require('http'),
	fs=require('fs'),
	url = require('url'),
	ws = require('websocket').server,
	crypto = require('crypto');

// Constantes
const MIME_TYPES={
	'html':'text/html',
	'js':'text/javascript',
	'manifest':'text/cache-manifest',
	'css': 'text/css',
	'png': 'image/png',
	'jpg': 'image/jpeg',
	'ico': 'image/ico',
	'mp3': 'audio/mp3',
	'ogg': 'audio/ogg',
	'mid': 'audio/x-midi',
	'json': 'application/json',
	'csv': 'text/csv',
	'webapp':'application/x-web-app-manifest+json'
};

// Variables
var racine=__dirname+'/www', // on ajoute la racine par défaut
	domaine='127.0.0.1',
	port=8124;

// on essaie de récupérer l'argument
if(process.argv[2])
	racine=process.argv[2];
// on vérifie si la racine du serveur
// est bien un dossier
if(!fs.statSync(racine).isDirectory())
	throw Error('La racine du serveur est mauvaise.');

var serveurHTTP=http.createServer(function (request, response) {
	// on parse l'url
	var urlParsée=url.parse(request.url);
	// on ne fait rien si c'est une connexion websocket
	if(urlParsée.pathname==='/ws')
		return;
	// contenu dynamique
	// liste des salons
	if(urlParsée.pathname==='/salons/liste.csv'&&
		(request.method=='HEAD'||request.method=='GET')) {
		// on envoie un code de succès
		response.writeHead(200,{'Content-Type':MIME_TYPES['csv']});
		// on itère sur le tableau des salons pour envoyer son contenu
		salons.forEach(function(salon){
			response.write(salon.id+',"'+salon.nom+'","'+salon.mode
				+'",'+salon.joueurs.length+'\n');
		});
		// on finalise la requête
		response.end();
		// on schinte le serveur statique
		return;
	}
	// liste des univers
	if(urlParsée.pathname==='/univers/liste.json'&&
		(request.method=='HEAD'||request.method=='GET')) {
		// On liste le contenu du dossier /univers
		fs.readdir(racine+'/univers',function(erreur,fichiers) {
			var index=-1;
			// en cas d'erreur, on stoppe tout
			if(erreur) {
				response.writeHead(500);
				response.end();
				throw Error('Impossible de lire le dossier des univers.');
			}
			// sinon, on envoie un code de succès
			response.writeHead(200,{'Content-Type':MIME_TYPES['json']});
			// on supprime les extensions de fichier
			fichiers.map(function(fichier,i) {
				fichiers[i]=fichier.replace(/.json$/,'');
				// on garde l'index du fichier liste éventuellement présent
				if('liste'==fichiers[i])
					index=i;
			});
			// si le fichier liste existe, on le supprime
			if(-1!==index)
				fichiers.splice(index,1);
			// on envoi le contenu en json
			response.end(JSON.stringify(fichiers));
		});
		// on schinte le serveur statique
		return;
	}
	/*/ génération du manifeste
	if(urlParsée.pathname==='/application.manifest'&&
		(request.method=='HEAD'||request.method=='GET')) {
		// Parallélisation des listing du contenu de dossiers
		var dossiers=['univers','javascript','images','sounds','css'];
		var listings=[];
		var dossiersRestants=dossiers.length;
		dossiers.forEach(function(nom) {
			fs.readdir(racine+'/'+nom,function(erreur,fichiers) {
				// en cas d'erreur, on stoppe tout
				if(erreur) {
					response.writeHead(500);
					response.end();
					throw Error('Impossible de lire le dossier "'+nom+'".');
				}
				// on garde une référence sur la liste de fichiers
				listings[nom]=fichiers;
				// si tous les dossiers ont été lus
				if(0==--dossiersRestants) {
					// sinon, on envoie un code de succès
					response.writeHead(200,{'Content-Type':MIME_TYPES['manifest']});
					// puis on génère le manifeste
					response.write('CACHE MANIFEST\n# v 1.0:'+process.pid+'\n\nCACHE:\n/index.html\n');
					// on itère sur chaque listing
					dossiers.forEach(function(nom) {
						// puis sur chaque fichier du listing
						for(var i=listings[nom].length-1; i>=0; i--) {
							// enfin on écrit la ligne du manifeste (sauf pour liste.json)
							// qui est dans la section fallback
							if(-1!==listings[nom][i].indexOf('.')&&'liste.json'!==listings[nom][i])
								response.write('/'+nom+'/'+listings[nom][i]+'\n');
						}
					});
					// on finalise le manifeste
					response.end('\nFALLBACK:\n/univers/liste.json /univers/liste.json\n\nNETWORK:\n*\n');
				}
			});
		});
		// on schinte le serveur statique
		return;
	}*/
	// contenu statique
	// accès en lecture uniquement
	if(request.method!='HEAD'&&request.method!='GET') {
		response.writeHead(401);
		response.end();
		return;
	}
	// vérification de l'existence
	// on refuse les paramètres d'URL
	if('search' in urlParsée) {
		response.writeHead(401);
		response.end();
	}
	// on redirige vers l'index si le client
	// atteint la racine du site
	if('/'===urlParsée.pathname||!urlParsée.pathname) {
		response.writeHead(301,{'Location':'/index.html'});
		response.end();
		return;
	}
	// On vérifie l'existence du fichier correspondant
	fs.stat(racine+urlParsée.pathname,
		function(erreur,resultat) {
			var headers={}, code=0, début=0, fin;
			// si une erreur survient
			// on envoi une erreur 404
			if(erreur||!resultat.isFile()) {
				response.writeHead(404);
				response.end();
				return;
			}
			// on trouve l'extension
			// si elle n'est pas supportée
			// on envoie une erreur 500
			// on stoppe le serveur
			var ext=urlParsée.pathname
				.replace(/^(?:.*)\.([a-z0-9]+)$/,'$1');
			if(!MIME_TYPES[ext]) {
				response.writeHead(500);
				response.end();
				throw Error('Fichier au type mime non-supporté');
			}
			headers['Content-Type']=MIME_TYPES[ext];
			headers['Content-Length']=resultat.size;
			// on vérifie si le contenu doit être morcelé
			if(request.headers.range) {
				// on supprime la chaîne bytes=
				// de l'entête Range et on récupère
				// un tableau contenant le début et la fin
				// de la plage d'octets
				var morceaux = request.headers.range.replace(/bytes=/, "").split("-");
				// on convertit la variable début en entier
				début = parseInt(morceaux[0],10);
				// si il y a une fin, on convertit en entier
				// sinon, la fin est la taille du fichier
				fin =  morceaux[1] ? parseInt(morceaux[1], 10) :
					headers['Content-Length']-1; 
				headers['Content-Range'] = 'bytes ' + début + '-' + fin + '/'
					+ (headers['Content-Length']);
				headers['Accept-Ranges'] = 'bytes';
				headers['Content-Length']= (fin-début)+1;
				headers['Transfer-Encoding'] = 'chunked';
				headers['Connection'] = 'close';
				code=206;
			}
		else
			code=200;
		// on envoi le code et les entêtes
		response.writeHead(code, headers);
		// si la méthode est GET alors on envoi
		// aussi le contenu du fichier
		if(request.method=='GET') {
			// on transvase le flux de lecture
			// en prenant soin d'indiquer la
			// plage d'octets concernés 
			fs.createReadStream(racine
			+urlParsée.pathname,{start: début, end: fin})
			.pipe(response);
		}
		else
			response.end();
	}); 
}).listen(port);

console.log('Serveur démarré sur http://'+domaine+':'+port+'/');
console.log('Racine du serveur : '+racine);

// variables temps réèl
// on crée le tableau des joueurs
var joueurs=[];
var idJoueurs=0;
// on crée également le stockage des connexions
var connexions={};
// on crée le tableau des salons avec deux salons dedans
var salons=[{'id':1,'nom':'Le bon coin','joueurs':[],'mode':'normal'},
	{'id':2,'nom':'Castagne','joueurs':[],'mode':'arcade'}],
	idSalons=2;
// on garde une liste des souscriptions aux salons
var souscriptions=[];
souscriptions[1]=[];
souscriptions[2]=[];

// ajout de salons fictifs
for(var i=8; i>=0; i--)
	salons.push(salons[i%2]);

// on crée le serveur websocket en lui passant
// une référence au serveur HTTP
var serveurWebsocket = new ws({
	httpServer: serveurHTTP,
	autoAcceptConnections: false
});

// On écoute les demandes de connexion
serveurWebsocket.on('request', function(request) {
	// Si l'origine de la connexion n'est pas notre site, on la rejette
	if(-1===request.origin.indexOf('http://'+domaine+':'+port)) {
		console.log(new Date()+': Origine de la connexion rejetée ('+request.origin+').');
		request.reject();
		return;
	}
	// on récupère l'objet connexion
   var connexion = request.accept(null, request.origin),
	// on crée les variables relatives au joueur
		joueur={}, sessid='';
   console.log((new Date()) + ': Nouvelle connexion acceptée.');
	// on écoute les messages reçus depuis cette connexion
  connexion.on('message', function(message) {
		// on crée une variable pour contenir le message décodé
		var contenu;
		// on ne traite que les message encodés en utf8
    if ('utf8' === message.type)	{
			// on parse le JSON reçu
			try {
				contenu=JSON.parse(message.utf8Data);
			} catch(e) {
				// si le le JSON est mal formé on ignore le message
				console.log('JSON mal formé: ' + message.utf8Data);
				return;
			}
			// on vérifie si le JSON a bien un type
			if(!contenu.type)
				return;
			// on choisi l'action à faire selon le type
			switch(contenu.type) {
				// VUE PROFIL
				case 'connexion':
					// on vérifie que les infos soient présentes
					if(!(contenu.nom&&contenu.image1&&contenu.image2))
						return;
					// si l'utilisateur a un id, on le cherche
					// dans la liste des connexions en sursis
					if(contenu.sessid&&connexions[contenu.sessid]) {
						sessid=contenu.sessid;
						joueur=connexions[contenu.sessid].joueur;
						// on ferme éventuellement l'ancienne connexion
						if(connexions[contenu.sessid].connexion)
							connexions[contenu.sessid].connexion.close();
						connexions[contenu.sessid].connexion=connexion;
						// on arrête le timer
						if(connexions[sessid].timeout) {
							clearTimeout(connexions[sessid].timeout);
							connexions[sessid].timeout=0;
						}
					}
					// sinon, on le crée
					else {
						var hash=crypto.createHash('sha1');
						hash.update(message.utf8Data,'utf8');
						sessid=hash.digest('hex');
						connexions[sessid]={'connexion':connexion,'joueur':joueur};
						joueur.id=++idJoueurs;
					}
					// on stocke les infos sur le joueur
					joueur.nom=(''+contenu.nom).trim().replace('&','&amp;').replace('<','&lt')
						.replace('>','&gt').replace('"','&quot;').trim();
					joueur.sexe=(contenu.sexe?(contenu.sexe===-1?-1:1):0);
					joueur.image1=contenu.image1;
					joueur.image2=contenu.image2;
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Connexion.');
					// enfin, on envoi l'id de la connexion
					connexion.sendUTF(JSON.stringify({'type':'connexion','sessid':sessid,'id':joueur.id}));
					break;
				// VUE SALONS
				case 'salons':
					// vérification du numéro de page
					if((!contenu.page)||Number.isNaN(parseInt(contenu.page,10))
						||salons.length<=(contenu.page-1)*10)
						return false;
					// envoi de la liste des salons par copie
					connexion.sendUTF(JSON.stringify({'type':'salons',
						'liste':salons.slice((contenu.page-1)*10,10).map(function(salon) {
							// on prends soin de réduire la liste des
							// joueurs au nombre de joueurs seulement
							return {'id':salon.id,'nom':salon.nom,'mode':salon.mode,
								'joueurs':salon.joueurs.length};
						})}));
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Listing.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				// VUE SALON
				case 'salon':
					// connexion à un salon
					// on itère sur tous les salons
					if(!(contenu.salon&&salons.some(function(salon) {
						// on quitte si ce n'est pas le bon salon
						// ou s'il est complet
						if(salon.id!=contenu.salon||salon.joueurs.length>2)
							return false
						// si l'utilisateur est déjà dans le salon on fait rien
						if(-1!==souscriptions[salon.id].indexOf(sessid))
							return true;
						connexion.sendUTF(JSON.stringify({'type':'salon',
							'salon':salon}));
						salon.joueurs.push(joueur);
						// notification des joueurs du salon
						souscriptions[salon.id].forEach(function(destId) {
							connexions[destId].connexion.sendUTF(JSON.stringify({'type':'arrivee','joueur':joueur}));
						});
						souscriptions[salon.id].push(sessid);
						connexions[sessid].salon=salon;
						return true;
					}))) {
						connexion.sendUTF(JSON.stringify({'type':'salon','salon':null}));
						// on retire le joueur du salon
						if(connexions[sessid].salon) {
							var index=connexions[sessid].salon.joueurs.indexOf(joueur);
							if(-1!==index)
								connexions[sessid].salon.joueurs.splice(index,1)
							souscriptions[connexions[sessid].salon.id].splice(
								souscriptions[connexions[sessid].salon.id].indexOf(sessid),1);
							// notification des joueurs du salon
							souscriptions[connexions[sessid].salon.id].forEach(function(destId) {
								connexions[destId].connexion.sendUTF(JSON.stringify({'type':'depart','joueur':joueur.id}));
							});
							connexions[sessid].salon=null;
						}
					}
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Salon.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				case 'chat':
					// message sur le chat
					if((!connexions[sessid].salon)||!contenu.message)
						return;
					// fitrage du html
					contenu.message=contenu.message.replace('&','&amp;').replace('<','&lt')
						.replace('>','&gt').replace('"','&quot;');
					// envoi aux joueurs du salon
					souscriptions[connexions[sessid].salon.id].forEach(function(destId) {
						connexions[destId].connexion.sendUTF(JSON.stringify({'type':'chat','joueur':joueur.nom,
							'message':contenu.message}));
						});
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Chat.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				// VUE NOUVEAUSALON
				case 'nouveau':
					// message sur le chat
					if(!contenu.nom)
						return;
					// fitrage du html
					contenu.nom=contenu.nom.replace('&','&amp;').replace('<','&lt')
						.replace('>','&gt').replace('"','&quot;');
					// ajout du salon
					salons.push({'id':++idSalons,'nom':contenu.nom,'joueurs':[],'mode':'normal'});
					souscriptions[idSalons]=[];
					// envoi de la confirmation
					connexion.sendUTF(JSON.stringify({'type':'nouveau'}));
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Nouveau salon'+contenu.nom+'.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				// VUE MULTI
				case 'debut':
					// on sonne le début de la partie
					if(connexions[sessid].salon&&connexions[sessid].salon.joueurs.length==3
						&&!connexions[sessid].salon.cartes) {
						var cartes=[];
						for(var i=2; i>=0; i--) {
							// on crée les cartes de chaque joueur
							for(var j=1; j>=0; j--) {
								cartes.push({'joueur':connexions[sessid].salon.joueurs[i],
									'numero':(j+1), 'statut':0});
								cartes.push({'joueur':connexions[sessid].salon.joueurs[i],
									'numero':(j+1), 'statut':0});
							}
							// on initialise le joueur
							connexions[sessid].salon.joueurs[i].score=0;
							connexions[sessid].salon.joueurs[i].essais=0;
							connexions[sessid].salon.joueurs[i].carte1=null;
						}
						// on distribue les cartes
						connexions[sessid].salon.cartes=[];
						while(cartes.length)
							connexions[sessid].salon.cartes.push(cartes.splice(
								Math.floor(Math.random()*cartes.length),1)[0]);
						// envoi aux joueurs du signal de début
						souscriptions[connexions[sessid].salon.id].forEach(function(destId) {
							connexions[destId].connexion.sendUTF(JSON.stringify({'type':'debut'}));
						});
					}
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Début.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				case 'retourne':
					// on vérifie la présence du numéro de carte
					if(!(contenu.carte&&connexions[sessid].salon
						&&connexions[sessid].salon.cartes
						&&connexions[sessid].salon.cartes[contenu.carte-1]))
						return;
					// si c'est la deuxième carte et que la paire est ok
					if(joueur.carte1&&joueur.carte1!=contenu.carte
						&&connexions[sessid].salon.cartes[joueur.carte1-1].numero
						==connexions[sessid].salon.cartes[contenu.carte-1].numero
						&&connexions[sessid].salon.cartes[joueur.carte1-1].joueur
						==connexions[sessid].salon.cartes[contenu.carte-1].joueur) {
						// Si la paire a pas été trouvée entre temps
						if(!connexions[sessid].salon.cartes[joueur.carte1-1].trouvée) {
							// envoi de la paire à tous les joueurs
							souscriptions[connexions[sessid].salon.id].forEach(function(destId) {
								connexions[destId].connexion.sendUTF(JSON.stringify({
									'type':'paire','gagnant':joueur.id,
									'joueur':connexions[sessid].salon.cartes[contenu.carte-1].joueur.id,
									'carte1':joueur.carte1,
									'carte2':contenu.carte,
									'numero':connexions[sessid].salon.cartes[contenu.carte-1].numero}));
							});
							connexions[sessid].salon.cartes[joueur.carte1-1].trouvée=true;
							connexions[sessid].salon.cartes[contenu.carte-1].trouvée=true;
							// incrémentation du score
							joueur.score+=1;
							joueur.carte1=null;
							// vérification de la présence de concurrents
							var joueurRestant=null;
							if(!connexions[sessid].salon.cartes.some(function(carte) {
								if(carte.trouvée)
									return false;
								if(!joueurRestant) {
									joueurRestant=carte.joueur.id;
									return false;
								}
								return carte.joueur.id!=joueurRestant;
							})) {
								// envoi les scores à tous les joueurs
								souscriptions[connexions[sessid].salon.id].forEach(function(destId) {
									connexions[destId].connexion.sendUTF(JSON.stringify({
										'type':'victoire','gagnant':joueurRestant,
										'scores':{}}));
								});
								// suppression des cartes
								connexions[sessid].salon.cartes=null;
							}
						}
					}
					// si la paire n'a pas encore été trouvée
					else if(!connexions[sessid].salon.cartes[contenu.carte-1].trouvée) {
						// si c'est la première carte
						if(!joueur.carte1) {
							joueur.carte1=contenu.carte
						} else {
							joueur.essais++;
							joueur.carte1=null;
						}
						// envoi de la carte
						connexion.sendUTF(JSON.stringify({'type':'carte','carte':contenu.carte,
							'joueur':connexions[sessid].salon.cartes[contenu.carte-1].joueur.id,
							'numero':connexions[sessid].salon.cartes[contenu.carte-1].numero}));
					} else {
						joueur.essais++;
						joueur.carte1=null;
					}
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Retourne.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				default:
					console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
						+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Autre.');
					console.log('Message reçu: ' + message.utf8Data);
					break;
				}
			}
		});
	// on écoute la fermeture des connexions
  connexion.on('close', function(reasonCode, description) {
	// si le client était connecté
		if(connexions[sessid]) {
			// on supprime la connexion au bout de 30 secondes
			connexions[sessid].timeout=setTimeout(function() {
		        console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
				+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Connexion supprimée.');
				// on le retire du salon dans lequel il était
				if(connexions[sessid].salon) {
					connexions[sessid].salon.joueurs.splice(
						connexions[sessid].salon.joueurs.indexOf(joueur),1);
					souscriptions[connexions[sessid].salon.id].splice(
						souscriptions[connexions[sessid].salon.id].indexOf(sessid),1);
					// notification des joueurs du salon
					souscriptions[connexions[sessid].salon.id].forEach(function(destId) {
						connexions[destId].connexion.sendUTF(JSON.stringify({'type':'depart','joueur':joueur.id}));
					});
				}
				// on supprime la connexion
				delete connexions[sessid];
			},30000);
		}
    console.log((new Date()) + ' ['+connexion.remoteAddress+'-'
			+(joueur?joueur.nom+'('+joueur.id+')':'')+']: Déconnexion.');
	});
});

console.log('Serveur websocket démarré.');
