window.onload=function() {
	console.log('test');
}

// pour analyse statique
function statique() {
	require(['VueSalons','VueProfil','VueOptions','VueSalons','VueSalon','VueMono','VueMaj','VueConnexion','VueNouveauSalon']);
}
// AMD + global
(function(root,define){ define('Application',['Vue','require',
		'libs/commandor/Commandor','libs/sounds/Sounds'],
	function (Vue,require,Commandor,Sounds) {

	// Objet application
	function Application(racine) {
		// Recherche d'une nouvelle version
		if(window.applicationCache) {
			window.applicationCache.addEventListener('updateready',function() {
				// on ne demande la mise à jour
				// que si le joueur n'est pas entrain de jouer
				if((!(root.VueMono||root.VueMulti))||this.vue instanceof root.VueMono
					||this.vue instanceof root.VueMulti)
					this.montrerVue('Maj');
			}.bind(this));
		}
		// stockage de l'élément HTML racine
		this.racine=racine;
		// création d'un gestionnaire de commande
		this.gestionnaire=new Commandor(racine);
		// création d'un gestionnaire de sons et ajout des sons
		this.sons=new Sounds('sounds',
			// joue le son de fond une fois chargé
			function() { this.sons.play('mountainman'); }.bind(this));
		this.sons.register('mountainman',['mid','mp3'],Infinity,0.2);
		this.sons.register('carte1',['ogg']);
		this.sons.register('carte2',['ogg']);
		this.sons.register('bien',['ogg','mp3']);
		this.sons.register('pasbien',['ogg']);
		this.sons.register('applaudissements',['ogg']);
		this.sons.register('alerte1',['ogg']);
		this.sons.register('alerte2',['ogg']);
		this.sons.register('perdu',['ogg']);
		this.sons.register('gagne',['ogg']);
		// Récupération de la préférence son coupé
		try {
			if(window.localStorage&&window.localStorage.sonCoupé)
				this.sons.mute(!!window.localStorage.sonCoupé);
		} catch(e) {}
		// Enregistrement de la commande de changement de vue
		this.gestionnaire.suscribe('changerVue',
			this.changerVue.bind(this));
		// stockage de l'élément pour afficher des messages
		this.message=document.querySelector('div.app div.message p');
		// on crée la variable contenant les essais websocket
		this.essaisWS=0;
		// on affiche la première vue
		this.montrerVue('Accueil');
	}

	// Gestion des messages
	Application.prototype.afficherMessage=function (texte,durée,callback) {
		if(this.messageTimeout)
			clearTimeout(this.messageTimeout);
		this.messageTimeout=setTimeout(this.cacherMessage.bind(this),durée||1500);
		this.messageCallback=callback;
		this.message.firstChild.textContent=texte;
		this.message.parentNode.classList.add('affiche');
	};

	Application.prototype.cacherMessage=function () {
		this.message.parentNode.classList.remove('affiche');
		this.messageTimeout=null;
		this.messageCallback&&this.messageCallback();
		this.messageCallback=null;
	};

	// Gestion des vues
	Application.prototype.changerVue=function (event,params) {
		this.montrerVue(params.vue);
	};

	Application.prototype.montrerVue=function (name) {
		// on désamorce la vue précédente
		if(this.vueAffichée)
			this.vueAffichée.uninit();
		// on lance la vue suivante
		// soit la vue a un constructeur dédié
		// soit on utilise le constructeur générique Vue
		console.log('Affichage vue '+name);
		// on teste d'abord le contexte global
		if(root.Vue) {
			if(root['Vue'+name])
				this.vueAffichée=new root['Vue'+name]();
			else
				this.vueAffichée=new Vue();
			this.vueAffichée.init(this,name);
		} else {
			// sinon, RequireJS
			require(['Vue'+name],function(VuePerso) {
				// succès
				this.vueAffichée=new VuePerso();
				this.vueAffichée.init(this,name);
			}.bind(this),function(){
				// échec
				this.vueAffichée=new Vue();
				this.vueAffichée.init(this,name);
			}.bind(this));
		}
	};

	// Gestion des websockets
	Application.prototype.connexion=function () {
		// on garde une référence au callback de message
		if(this.connexionWS&&this.connexionWS.onmessage)
			var retourMessage=this.connexionWS.onmessage;
		// si on a 3 essais de connexion infructueux
		if(this.essaisWS>2) {
			// on annule tout
			this.connexionWS.close();
			this.connexionWS=null;
			this.essaisWS=0;
			// on affiche la vue erreur réseau
			this.montrerVue('Reseau');
		}
		// sinon, on retente une connexion
		this.essaisWS++;
		this.connexionWS = new WebSocket('ws://'+document.location.hostname+':8124');
		// on crée un timer pour abandonner la connexion
		// au bout de 5 secondes
		this.timeout=setTimeout(function() {
			if(this.essaisWS>0) {
				this.connexionWS.close();
				this.connexion();
			}
		}.bind(this),5000);
		// à l'ouverture de la connexion, on envoi les infos
		this.connexionWS.onopen = function () {
			if(retourMessage)
				this.connexionWS.onmessage=retourMessage;
			// on supprime le timeout
			if(this.timeout)
				clearTimeout(this.timeout);
			// on remet à zéro les essais
			this.essaisWS=0;
			// on envoi les infos utilisateur
			this.identification();
		}.bind(this);
		// on logge les erreurs pour débogage
		this.connexionWS.onerror = function (error) {
			console.log(error);
		}.bind(this);
		// on écoute toute fermeture
		this.connexionWS.onclose = function () {
			// on supprime le timeout
			if(this.timeout)
				clearTimeout(this.timeout);
			// On tente une reconnexion
			setTimeout(this.connexion.bind(this),2000);
		}.bind(this);
		// on écoute toute fermeture
		this.connexionWS.addEventListener('message', function (event) {
			console.log(event.data);
		});
	};

	// méthode d'envoi du profil
	Application.prototype.identification=function () {
		this.connexionWS.send(JSON.stringify({
			'type':'connexion',
			'nom':this.utilisateur.nom,
			'sexe':this.utilisateur.sexe,
			'image1':this.utilisateur.image1,
			'image2':this.utilisateur.image2,
			'sessid':(this.utilisateur.sessid||'')
		}));
	};

	// méthode de déconnexion
	Application.prototype.déconnexion=function () {
		// on supprime l'écouteur d'évènement
		this.connexionWS.onclose=null;
		// on ferme la connexion
		this.connexionWS.close();
		// on supprime toute référence à la connexion
		this.connexionWS=null;
	};

	// On lance l'application
	new Application(document.querySelector('div.app'));

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});
