if (!Function.prototype.bind) {
  Function.prototype.bind = function (oThis) {
    if (typeof this !== "function") {
      // closest thing possible to the ECMAScript 5 internal IsCallable function
      throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
    }

    var aArgs = Array.prototype.slice.call(arguments, 1), 
        fToBind = this, 
        fNOP = function () {},
        fBound = function () {
          return fToBind.apply(this instanceof fNOP && oThis
                                 ? this
                                 : oThis,
                               aArgs.concat(Array.prototype.slice.call(arguments)));
        };

    fNOP.prototype = this.prototype;
    fBound.prototype = new fNOP();

    return fBound;
  };
}

// AMD + global
(function(root,define){ define('Vue',[], function () {

	// Vues génériques
	function Vue() {}

	// Initialisation de la vue
	Vue.prototype.init=function (app,nom) {
		// on garde une référence à l'application
		this.app=app;
		// on sauve le nom de la vue
		this.nom=nom;
		// récupération de l'élément HTML correspondant
		this.contenu=document.getElementById(nom);
		if(!this.contenu)
			throw Error('La vue "'+nom+'" n\'a pas été trouvée.');
		// on affiche la vue
		this.contenu.classList.add('choisie');
	};

	// Enregistrement de commande simplifié
	Vue.prototype.commande=function (nom,méthode) {
		// si pas de méthode spécifiée :
		// on utilise le nom de la commande
		méthode=méthode||nom;
		if(!this[méthode])
			throw Error('Impossible de trouver le méthode "'+méthode+'".');
		// on enregistre la méthode dans le gestionnaire
		this.app.gestionnaire.suscribe(this.nom+'/'+nom,
			this[méthode].bind(this));
	};

	// Désarmorçage de la vue
	Vue.prototype.uninit=function () {
		// Suppression des commandes
		this.app.gestionnaire.unsuscribe(this.nom);
		this.contenu.classList.remove('choisie');
	};

	return Vue;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('libs/commandor/Commandor',[], function() {

	// Commandor constructor : rootElement is the element
	// from wich we capture commands
	var Commandor=function Commandor(rootElement) {
		if((!rootElement)||! (rootElement instanceof HTMLElement))
			throw Error('No rootElement given');
		// Commands hashmap
		this.commands={};
		// Helper for event listening
		rootElement.addEvent=rootElement.addEventListener||rootElement.attachEvent;
		// keeping a reference to the rootElement
		this.rootElement=rootElement;
		// MS Pointer events
		if(!!('onmsgesturechange' in window)) {
			// event listeners for buttons
			(function() {
				var curElement=null;
					this.rootElement.addEvent('MSPointerDown', function(event) {
					curElement=this.findButton(event.target);
					}.bind(this),true);
				this.rootElement.addEvent('MSPointerUp', function(event) {
					if(curElement==this.findButton(event.target))
						this.captureButton(event);
					else
						curElement=null;
					}.bind(this),true);
			}).call(this);
		} else {
			// Webkit touch events
			if(!!('ontouchstart' in window)) {
				(function() {
					var curElement=null;
					// variable contenant le dernier élément visé
					this.rootElement.addEvent('touchstart', function(event) {
						curElement=this.findButton(event.target);
						}.bind(this),true);
					this.rootElement.addEvent('touchend', function(event) {
						// on vérifie qu'on est toujours sur le même élément
						if(curElement==this.findButton(event.target))
							this.captureButton(event);
						// sinon, on vide la variable
						else
							curElement=null;
					}.bind(this),true);
				}).call(this);
			}
		// Clic events
		this.rootElement.addEvent('click',
			this.captureButton.bind(this),true);
		}
	// event listeners for forms submission
	this.rootElement.addEvent('submit',
		this.captureForm.bind(this),true);
	// event listeners for form changes
	this.rootElement.addEvent('change',
		this.formChange.bind(this),true);
	this.rootElement.addEvent('input',
		this.formChange.bind(this),true);
	this.rootElement.addEvent('select',
		this.formChange.bind(this),true);
	};

	// Look for a button
	Commandor.prototype.findButton=function(element) {
		// on remonte les parent de l'élément pour
		// trouver un lien ou un bouton de soumission
		while(element!==this.rootElement
			&&element.nodeName!=='A'
			&&(element.nodeName!='INPUT'
					||(!element.hasAttribute('type'))
					||element.getAttribute('type')!='submit'
					||!element.hasAttribute('formaction')))
			{
			element=element.parentNode;
			}
		return element;
		};

	// Button command handler
	Commandor.prototype.captureButton=function(event) {
		var element=this.findButton(event.target),
			command='';
		// looking for a button with formaction attribute
		if('INPUT'===element.nodeName
				&&element.hasAttribute('type')
				&&element.getAttribute('type')=='submit'
				&&element.hasAttribute('formaction'))
			command=element.getAttribute('formaction');
		// looking for a link
		if('A'===element.nodeName&&element.hasAttribute('href'))
			command=element.getAttribute('href');
		// executing the command
		this.executeCommand(event,command,element);
		command&&(event.stopPropagation()||event.preventDefault());
	};

	// Form change handler
	Commandor.prototype.formChange=function(event) {
		var element=event.target,
			command='';
		// find the evolved form
		while(element!==this.rootElement
			&&'FORM'!==element.nodeName) {
			element=element.parentNode;
		}
		// searching the data-change attribute containing the command
		if('FORM'===element.nodeName&&element.hasAttribute('data-change'))
			command=element.getAttribute('data-change');
		// executing the command
		this.executeCommand(event,command,element);
	};

	// Form command handler
	Commandor.prototype.captureForm=function(event) {
		var element=event.target,
			command='';
		while(element!==this.rootElement
			&&'FORM'!==element.nodeName) {
			element=element.parentNode;
			}
		// Si l'élément est un formulaire et propose une commande
		if('FORM'===element.nodeName&&element.hasAttribute('action'))
			command=element.getAttribute('action');
		// Si la commande existe on stoppe la propagation
		// et on annule l'évènement
		console.log('form'+command);
		this.executeCommand(event,command,element);
		command&&(event.stopPropagation()
			||event.preventDefault());
	};

	// Common command executor
	Commandor.prototype.executeCommand=function (event,command,element) {
		// checking for the app protocol
		if(0!==command.indexOf('app:'))
			return false;
		// removing app:
		command=command.substr(4);
		var chunks=command.split('?');
		// the first chunk is the command path
		var callback=this.commands;
		var nodes=chunks[0].split('/');
		for(var i=0, j=nodes.length; i<j; i++) {
		if(!callback[nodes[i]])
			return false;
			callback=callback[nodes[i]];
		}
		// Preparing arguments
		var args=[];
		if(chunks[1]) {
			chunks=chunks[1].split('&');
			for(i=0, j=chunks.length; i<j; i++) {
				var parts=chunks[i].split('=');
				if(undefined!==parts[0]&&undefined!==parts[1])
					args[parts[0]]=decodeURIComponent(parts[1]);
			}
		}
		// executing the command fallback
		return !!!callback(event,args,element);
	};

	// Add a callback for the specified path
	Commandor.prototype.suscribe=function(path,callback) {
		var nodes=path.split('/'),
			command=this.commands;
		for(var i=0, j=nodes.length-1; i<j; i++) {
			if((!command[nodes[i]])||!(command[nodes[i]] instanceof Object)) {
				command[nodes[i]]={};
				}
			command=command[nodes[i]];
			}
		command[nodes[i]]=callback;
	};

	// Delete callback for the specified path
	Commandor.prototype.unsuscribe=function(path) {
		var nodes=path.split('/'),
			command=this.commands;
		for(var i=0, j=nodes.length-1; i<j; i++) {
			command=command[nodes[i]]={};
		}
		command[nodes[i]]=null;
	};

	return Commandor;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	if(typeof name === 'Object') {
		deps=name; factory=deps;
	}
	this.Commandor=factory.apply(this, deps.map(function(){
		return this[deps.substring(deps.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('libs/sounds/Sounds',[], function() {

	// HTML5 Sounds manager
	function Sounds(folder,loadCallback) {
		if(!folder)
			throw new Error('No folder given for sounds !');
		// sound is on by default
		this.muted=false;
		// contains sounds elements
		this.sounds={};
		// contains sounds to load
		this.soundsToLoad=[];
		// callback executed when each sounds are loaded
		this.loadedSounds=loadCallback;
		// detecting supported extensions
		var sound=document.createElement('audio');
		this.exts=[];
		if(!sound.canPlayType)
			return;
		if(sound.canPlayType('audio/ogg'))
			this.exts.push('ogg');
		if(sound.canPlayType('audio/mp3'))
			this.exts.push('mp3');
		if(sound.canPlayType('audio/x-midi'))
			this.exts.push('mid');
		// folder containing sounds
		this.folder=folder;
	}

	// register a sound to load
	Sounds.prototype.register = function(name, extensions, iterations, volume) {
		if(!this.exts.length)
			return;
		// creating the Audio element
		var sound=new Audio();
		// Add in the sounds to load list
		this.soundsToLoad.push(sound);
		sound.setAttribute('preload','auto');
		if(extensions.every(function(ext) {
			if(-1===this.exts.indexOf(ext))
				return true;
			sound.setAttribute('src',this.folder+'/'+name+'.'+ext);
			return false;
		}.bind(this)))
			return;
		// iterating as needed
		if(Infinity===iterations)
			sound.setAttribute('loop','loop');
		else if(iterations>1)
			sound.setAttribute('data-iterations',iterations);
		if(volume)
			sound.setAttribute('data-volume',volume);
		sound.setAttribute('data-name',name);
		// adding callback
		sound.addEventListener('canplaythrough', this.soundLoaded.bind(this));
	},

	// remove soundsToLoad when loaded
	Sounds.prototype.soundLoaded = function(event) {
		// getting the index of the loaded sound
		var index=this.soundsToLoad.indexOf(event.target);
		if(index>=0) {
			var sound=this.soundsToLoad.splice(index,1)[0];
			this.sounds[sound.getAttribute('data-name')]=sound;
		}
		// if no more sounds to load, execute callback
		if(this.loadedSounds&&!this.soundsToLoad.length)
			this.loadedSounds();
	};

	// Play a sound
	Sounds.prototype.play = function(name,iterations) {
		// if the sound exists
		if(this.sounds[name]&&(this.sounds[name].hasAttribute('loop')||!this.muted)) {
			// getting iteration count
			if(!iterations) {
				iterations=1;
				if(this.sounds[name].hasAttribute('data-iterations'))
					iterations=parseInt(
						this.sounds[name].getAttribute('data-iterations'),10);
			}
			// cloning the node and playing the sound
			this.sounds[name].currentlyPlayed=
				this.sounds[name].cloneNode();
			if(this.sounds[name].hasAttribute('data-volume')) {
				this.sounds[name].currentlyPlayed.volume=
					this.sounds[name].currentlyPlayed
						.getAttribute('data-volume');
			}
			// if the sound is not muted and it's a background sound
			if(!this.muted) {
				this.sounds[name].currentlyPlayed.play();
				this.sounds[name].currentlyPlayed
					.addEventListener('ended', function() {
					if(--iterations)
						this.sounds[name].currentlyPlayed.play();
					else
						this.sounds[name].currentlyPlayed=null;
				}.bind(this));
			}
		}
	};

	// Stops a sound
	Sounds.prototype.stop = function(name) {
		if(this.sounds[name].currentlyPlayed)
			this.sounds[name].currentlyPlayed.pause();
		this.sounds[name].currentlyPlayed=null;
	};

	// Mutes a sound
	Sounds.prototype.mute = function(muted) {
		for(var name in this.sounds) {
			if(this.sounds[name].currentlyPlayed) {
				this.sounds[name].currentlyPlayed[muted?'pause':'play']();
			}
		}
		this.muted=muted;
	};

	return Sounds;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	if(typeof name === 'Object') {
		deps=name; factory=deps;
	}
	this.Sounds=factory.apply(this, deps.map(function(){
		return this[deps.substring(deps.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueSalons',['Vue'], function (Vue) {

	function VueSalons() { }

	VueSalons.prototype=new Vue();

	VueSalons.prototype.init=function (app,name) {
		// Appel de la fonction parente
		Vue.prototype.init.bind(this)(app,name);
		// Enregistrement des commandes de la vue
		this.commande('rejoindre');
		this.commande('rafraichir');
		// Acquisition d'une référence aux éléments d'affichage
		this.tableau=document.querySelector('div.vue.choisie table tbody');
		this.modeleLigne=this.tableau.firstChild;
		this.tableau.removeChild(this.modeleLigne);
		// ajout de l'écouteur de messages websockets
		this.app.connexionWS.onmessage=this.message.bind(this);
		// envoi d'une demande de listing des salons
		this.rafraichir();
	};

	VueSalons.prototype.rejoindre=function (event, params) {
		this.app.connexionWS.send(JSON.stringify({'type':'salon','salon':params.salon}));
	};

	VueSalons.prototype.rafraichir=function (event, params) {
		// on vide la liste des salons
		this.vider();
		// on crée la requête
		var requête=new XMLHttpRequest();
		var indexDernièreLigne=0;
		requête.open('GET','/salons/liste.csv',true);
		requête.onprogress=requête.onload=function(event){
			// récupération du dernier retour à la ligne
			var nouvelIndex=requête.responseText.lastIndexOf('\n');
			// extraction des lignes
			if(nouvelIndex!==-1&&indexDernièreLigne<nouvelIndex) {
				var contenu=requête.responseText.substring(indexDernièreLigne,nouvelIndex+1);
				// parsing du CSV
				var valeur='';
				var champs=[];
				for(var i=0, j=contenu.length; i<j; i++) {
					// caractère d'échappement
					if('\\'==contenu[i]) {
						valeur+=contenu[i+1];
						i++;
					// caractères de fin de champs
					} else if(','==contenu[i]||'\r'==contenu[i]||'\n'==contenu[i]||i==j-1) {
						champs.push(valeur);
						valeur='';
						// caractère de fin de ligne
						if('\r'==contenu[i]||'\n'==contenu[i]||i==j-1) {
							// affichage de la ligne
							this.afficherLigne(champs);
							// raz des champs
							champs=[];
						if('\r'==contenu[i])
							i++;
						}
					} else {
						if(contenu[i]!='"')
							valeur+=''+contenu[i];
					}
				}
				// sauvegarde du nouvel index
				indexDernièreLigne=nouvelIndex+1;
			}
		}.bind(this);
		requête.send();
	};

	VueSalons.prototype.afficherLigne=function (champs) {
			var nouvelleLigne=this.modeleLigne.cloneNode(true);
			// Mise en place du lien
			nouvelleLigne.firstChild.firstChild.firstChild.textContent=champs[1];
			nouvelleLigne.firstChild.firstChild.setAttribute('href',
				this.modeleLigne.firstChild.firstChild.getAttribute('href')+champs[0]);
			// ajout du nombre de joueurs
			nouvelleLigne.childNodes[1].firstChild.textContent=champs[3]+'/3';
			// ajout du type de salon
			nouvelleLigne.childNodes[2].firstChild.textContent=champs[2];
			// insertion de la ligne
			this.tableau.appendChild(nouvelleLigne);
	};

	VueSalons.prototype.message=function (event) {
		var réponse=JSON.parse(event.data);
		if((!réponse.type))
			throw Error('Message innatendu de la part du serveur.');
		if('salon'==réponse.type) {
			// si le salon est tjrs ok, on charge la vue
			if(réponse.salon) {
				// on garde une référence au salon
				this.app.salon=réponse.salon;
				// on ajoute le joueur au salon
				this.app.salon.joueurs.push(this.app.utilisateur);
				// on affiche la vue salon
				this.app.montrerVue('Salon');
			}
			// sinon on, actualise
			else
				this.rafraichir();
		}
	};
	
	VueSalons.prototype.vider=function (event) {
		while(this.tableau.firstChild)
			this.tableau.removeChild(this.tableau.firstChild);
	};

	VueSalons.prototype.uninit=function (app) {
		this.vider();
		this.tableau.appendChild(this.modeleLigne);
		Vue.prototype.uninit.bind(this)();
	};

	return VueSalons;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueProfil',['Vue'], function (Vue) {

	function VueProfil() { }

	VueProfil.prototype=new Vue();

	VueProfil.prototype.init=function (app,name) {
		Vue.prototype.init.bind(this)(app,name);
		// création de l'utilisateur
		if(!this.app.utilisateur)
			this.app.utilisateur={'nom':'','sexe':-1};
		// Enregistrement des commandes
		this.commande('envoi');
		this.commande('changement');
		this.commande('photo');
		// Récupération du formulaire
		var form=document.querySelector('div.vue.choisie form');
		// Création de la tâche de fond
		this.tâcheFond=new Worker('/javascript/Compare.js');
		this.photosSeRessemblent=true;
		this.tâcheFond.onmessage=function(event) {
			if('ok'==event.data) {
				form.elements.photo2.setCustomValidity('');
			} else if('pasok'==event.data) {
				form.elements.photo2.setCustomValidity('Les images se ressemblent trop, merci d\'en choisir une autre.');
			}
		}.bind(this);
		// ajout de l'image par défaut
		var images=document.querySelectorAll('div.vue.choisie img.profil');
		// Ajout de l'évènement de dépôt
		form.addEventListener('dragenter', function(e) {
			e.stopPropagation(); e.preventDefault(); }, false);
		form.addEventListener('dragover', function(e) {
			e.stopPropagation(); e.preventDefault(); }, false);
		form.addEventListener('drop', this.dépôt.bind(this), false);
		// création de l'image par défaut
		this.imageDéfaut(images[0],form.elements.utilisateur.value,1);
		this.imageDéfaut(images[1],form.elements.utilisateur.value,2);
		// Tentative d'accès à la caméra
		(navigator.getUserMedia||navigator.webkitGetUserMedia
			||navigator.mozGetUserMedia||navigator.msGetUserMedia
			||(function(){})).call(navigator,{'video':true},
			this.initialiserVidéo.bind(this),
			function(erreur) {
				console.log(erreur);
			});
	};

	VueProfil.prototype.changement=function (event,params,form) {
		var images=document.querySelectorAll('div.vue.choisie img.profil');
		// on dessine l'image par défaut si il n'y a pas de photo
		if(event.target==form.elements.photo||(event.target==form.elements.utilisateur
			&&!(form.elements.photo.files&&form.elements.photo.files.length))) {
			if(form.elements.photo.files&&form.elements.photo.files.length)
				this.imagePerso(form.elements.photo.files[0],images[0],1);
			else
				this.imageDéfaut(images[0],form.elements.utilisateur.value,1);
			form.elements.photo2.setCustomValidity('Patientez pendant l\'analyse de votre image.');
		}
		if(event.target==form.elements.photo2||(event.target==form.elements.utilisateur
			&&!(form.elements.photo2.files&&form.elements.photo2.files.length))) {
			if(form.elements.photo2.files&&form.elements.photo2.files.length)
				this.imagePerso(form.elements.photo2.files[0],images[1],2);
			else
				this.imageDéfaut(images[1],form.elements.utilisateur.value,2);
			form.elements.photo2.setCustomValidity('Patientez pendant l\'analyse de votre image.');
		}
	};

	VueProfil.prototype.imagePerso=function (fichier,image,n) {
		// on récupère le contenu du fichier sous forme de dataUri
		var reader = new FileReader();
		reader.readAsDataURL(fichier);
		reader.onload = function(event) {
			// on charge la dataUri dans une image
			var img=document.createElement('img');
			img.src = event.target.result;
			// quand elle est chargée
			img.onload=function() {
				// on crée notre élément canvas
				var canvas=document.createElement('canvas');
				// on choisit les dimensions max de l'image
				canvas.width=90;
				canvas.height=90;
				// on récupère le contexte 2d
				var contexte=canvas.getContext('2d');
				// on dessine l'image dans le canvas
				// en prenant le plus grand carré possible
				// et en le centrant
				contexte.drawImage(img,
					(img.width>img.height?(img.width-img.height)/2:0),
					(img.width>img.height?0:(img.height-img.width)/2),
					(img.width>img.height?img.height:img.width),
					(img.width>img.height?img.height:img.width),
					0, 0,90,90);
				// on récupère le contenu du canvas sous forme de dataURI
				// pour le réinjecter dans l'image
				image.src = canvas.toDataURL();
				// puis on l'envoi à la tâche de fond pour traitement
				this.tâcheFond.postMessage(JSON.stringify({'image':n,'données':
					Array.prototype.slice.call(contexte.getImageData(0,0,90,90).data,0)}));
			}.bind(this);
		}.bind(this);
	};

	VueProfil.prototype.imageDéfaut=function (image,utilisateur,n) {
		// on crée notre élément canvas
		var canvas=document.createElement('canvas');
		// on choisit les dimensions de l'image
		canvas.width=90;
		canvas.height=90;
		// on récupère le contexte 2d
		var contexte=canvas.getContext('2d');
		// on dessine l'image dans le canvas
		// choix d'une couleur de remplissage aléatoire
		contexte.fillStyle = 'rgba('+Math.floor(Math.random()*200)
			+','+Math.floor(Math.random()*200)
			+','+Math.floor(Math.random()*200)+',0.4)';
		// dessin cercle 0
		contexte.beginPath();
		contexte.arc(Math.floor(Math.random()*80)+5,
			Math.floor(Math.random()*80)+5, 5, 0, Math.PI*2, true); 
		contexte.closePath();
		contexte.fill();
		// dessin cercle 1
		contexte.beginPath();
		contexte.arc(Math.floor(Math.random()*70)+10,
			Math.floor(Math.random()*70)+10, 10, 0, Math.PI*2, true); 
		contexte.closePath();
		contexte.fill();
		// dessin cercle 2
		contexte.beginPath();
		contexte.arc(Math.floor(Math.random()*60)+15,
			Math.floor(Math.random()*60)+15, 15, 0, Math.PI*2, true); 
		contexte.closePath();
		contexte.fill();
		// dessin cercle 3
		contexte.beginPath();
		contexte.arc(Math.floor(Math.random()*50)+20,
				Math.floor(Math.random()*50)+20, 20, 0, Math.PI*2, true); 
		contexte.closePath();
		contexte.fill();
		// on ajout le nom de l'utilisateur
		contexte.fillStyle = '#000';
		contexte.textAlign = 'center';
		contexte.textBaseline='middle';
		contexte.font = "20pt Arial";
		contexte.fillText(utilisateur, 45, (n==1?22:67),90);
		// on récupère le contenu du canvas sous forme de dataURI
		image.src = canvas.toDataURL();
		// puis on l'envoi à la tâche de fond pour traitement
		this.tâcheFond.postMessage(JSON.stringify({'image':n,'données':
			Array.prototype.slice.call(contexte.getImageData(0,0,90,90).data,0)}));
	};

	// fonction de réception de dépôt de fichier
	VueProfil.prototype.dépôt=function (event) {
		var images=document.querySelectorAll('div.vue.choisie img.profil');
		if(event.target==images[0]||event.target==images[1]) {
			if(event.dataTransfer.files&&event.dataTransfer.files.length
				&&event.dataTransfer.files[0].type.indexOf('image/')===0) {
					this.imagePerso(event.dataTransfer.files[0],event.target);
					var form=document.querySelector('div.vue.choisie form');
					form.elements.photo2.setCustomValidity('Patientez pendant l\'analyse de votre image.');
				}
		}
		// on annule l'action par défaut
		event.stopPropagation();
		event.preventDefault();
	};

	// On initialise l'interface utilisateur pour permettre
	// l'affichage de la webcam et la prise de photos
	VueProfil.prototype.initialiserVidéo=function (fluxVidéo) {
		var images=document.querySelectorAll('div.vue.choisie img.profil');
		var video = document.createElement('video');
		video.setAttribute('class','profil');
		if (navigator.mozGetUserMedia)
			video.mozSrcObject = fluxVidéo;
		else
			video.src = (window.URL || window.webkitURL)
				.createObjectURL(fluxVidéo);
		var a=document.createElement('a');
		a.setAttribute('title','Prendre une photo');
		var a2=a.cloneNode();
		a.setAttribute('href','app:Profil/photo?image=1');
		a.appendChild(video);
		video.play();
		a2.setAttribute('href','app:Profil/photo?image=2');
		a2.appendChild(video.cloneNode());
		a2.firstChild.play();
		images[0].parentNode.insertBefore(a,images[0]);
		images[1].parentNode.insertBefore(a2,images[1]);
	};

	VueProfil.prototype.photo=function (event, params) {
		var images=document.querySelectorAll('div.vue.choisie img.profil');
		var videos=document.querySelectorAll('div.vue.choisie video.profil');
		// on crée notre élément canvas
		var canvas=document.createElement('canvas');
		// on choisit les dimensions de l'image
		canvas.width=90;
		canvas.height=90;
		// on récupère le contexte 2d
		var contexte=canvas.getContext('2d');
		// on dessine l'image actuelle
		// du flux vidéo dans le canvas
		var vid=videos[params.image-1];
		contexte.drawImage(vid,
			(vid.videoWidth>vid.videoHeight?(vid.videoWidth-vid.videoHeight)/2:0),
			(vid.videoWidth>vid.videoHeight?0:(vid.videoHeight-vid.videoWidth)/2),
			(vid.videoWidth>vid.videoHeight?vid.videoHeight:vid.videoWidth),
			(vid.videoWidth>vid.videoHeight?vid.videoHeight:vid.videoWidth),
			0, 0,90,90);
		// on récupère le contenu du canvas sous forme de dataURI
		// pour le réinjecter dans l'image
		images[params.image-1].src = canvas.toDataURL();
	};

	VueProfil.prototype.envoi=function (event, params) {
	console.log('envoi');
		var images=document.querySelectorAll('div.vue.choisie img.profil');
		// on récupère les infos sur l'utilisateur
		this.app.utilisateur={'nom':event.target[0].value,
			'sexe':(event.target[4].checked?1:event.target[5].checked?0:-1),
			'image1':images[0].src,'image2':images[1].src};
		// on se connecte au serveur web socket
		if(!this.app.connexionWS)
			this.app.connexion();
		else
			this.app.identification();
		// on récupérère les messages
		this.app.connexionWS.onmessage=this.connecté.bind(this);
	};

	VueProfil.prototype.connecté=function (event) {
		var réponse=JSON.parse(event.data);
		if((!réponse.type)||'connexion'!=réponse.type||!réponse.sessid)
			throw Error('Message innatendu de la part du serveur.');
		// on stocke l'id du joueur
		this.app.utilisateur.sessid=réponse.sessid;
		this.app.utilisateur.id=réponse.id;
		// on affiche la vue salons
		this.app.montrerVue('Salons');
	};

	VueProfil.prototype.uninit=function (app) {
		var videos=document.querySelectorAll('div.vue.choisie video.profil');
		if(videos&&videos.length) {
			for(var i=videos.length-1; i>=0; i--)
				videos[i].parentNode.parentNode.removeChild(videos[i].parentNode);
		}
		Vue.prototype.uninit.bind(this)();
	};

	return VueProfil;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueOptions',['Vue'], function (Vue) {

	function VueOptions() { }

	VueOptions.prototype=new Vue();

	VueOptions.prototype.init=function (app,name) {
		Vue.prototype.init.bind(this)(app,name);
		this.commande('envoi');
		document.getElementById('sons').value=(this.app.sons.coupé?0:1);
		document.getElementById('sons').checked=(document.fullscreenElement
			||document.mozFullScreenElement||document.webkitFullscreenElement);
	};

	VueOptions.prototype.envoi=function (event, params) {
		var muter=(parseInt(event.target[0].value,10)?false:true);
		this.app.sons.mute(muter);
		try {
			if(window.localStorage)
				window.localStorage.sonCoupé=(muter?'true':'');
		} catch(e) {}
		// Si plein écran on l'active
		if(event.target[1].checked) {
			// on vérifie si le plein écran n'est pas déjà activé
			if((!document.fullscreenElement)&&(!document.mozFullScreenElement)
				&&!document.webkitFullscreenElement) {
				// auquel cas, on passe au mode plein écran
				if(document.documentElement.requestFullscreen)
					document.documentElement.requestFullscreen();
				else if(document.documentElement.mozRequestFullScreen)
					document.documentElement.mozRequestFullScreen();
				else if(document.documentElement.webkitRequestFullscreen)
					document.documentElement.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
			}
		}
		// on vérifie si le plein écran est activé
		else if(document.fullscreenElement||document.mozFullScreenElement
			||document.webkitFullscreenElement) {
			// auquel cas, on passe enlève le mode plein écran
			if(document.cancelFullScreen)
				document.cancelFullScreen();
			else if(document.mozCancelFullScreen)
				document.mozCancelFullScreen();
			else if(document.webkitCancelFullScreen)
				document.webkitCancelFullScreen();
		}
		this.app.montrerVue('Accueil');
	};

	return VueOptions;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueSalon',['Vue'], function (Vue) {

	function VueSalon()	{ }

	VueSalon.prototype=new Vue();

	VueSalon.prototype.init=function (app,name) {
		// Appel de la fonction parente
		Vue.prototype.init.bind(this)(app,name);
		// référencement des zone dynamiques
		this.h1=document.querySelector('div.vue.choisie h1');
		this.h1.firstChild.textContent=this.app.salon.nom+' ';
		this.bouton=document.querySelector('div.vue.choisie a[disabled="disabled"]');
		this.majJoueurs();
		this.chat=document.querySelector('div.vue.choisie p.chat');
		this.champs=document.querySelector('div.vue.choisie input[type="text"]');
		// Enregistrement des commandes de la vue
		this.commande('quitter');
		this.commande('envoi');
		this.commande('jouer');
		this.commande('notification');
		// vérification de la possibilité d'activer les notifications
		this.boutonNotification=document.querySelector('div.vue.choisie p.menu a:nth-child(3)');
		if('webkitNotifications' in window
			&&window.webkitNotifications.checkPermission()===1) {
				this.boutonNotification.style.display='';
			} else {
				this.boutonNotification.style.display='none';
			}
		// ajout de l'écouteur de messages websockets
		this.app.connexionWS.onmessage=this.message.bind(this);
	};

	VueSalon.prototype.quitter=function (event, params) {
		this.app.connexionWS.send(JSON.stringify({'type':'salon','salon':null}));
		this.app.salon=null;
	};

	VueSalon.prototype.envoi=function (event, params) {
		this.app.connexionWS.send(JSON.stringify({'type':'chat','message':this.champs.value}));
		this.champs.value='';
	};

	VueSalon.prototype.jouer=function (event) {
		this.app.connexionWS.send(JSON.stringify({'type':'debut'}));
	};

	VueSalon.prototype.message=function (event) {
		var réponse=JSON.parse(event.data);
		if((!réponse.type))
			throw Error('Message innatendu de la part du serveur.');
		if('debut'==réponse.type) {
			this.app.sons.jouer('gagne');
			this.app.montrerVue('Multi');
		} else if('salon'==réponse.type) {
			this.app.montrerVue('Salons');
		} else if('arrivee'==réponse.type) {
			this.app.salon.joueurs.push(réponse.joueur);
			this.ligneChat('',réponse.joueur.nom+' a rejoint le salon.');
			this.majJoueurs();
		} else if('depart'==réponse.type) {
			this.app.salon.joueurs.some(function(joueur,i) {
				if(réponse.joueur==joueur.id) {
					this.app.salon.joueurs.splice(i,1);
					this.ligneChat('',joueur.nom+' a quitté le salon.');
					return true;
				}
			}.bind(this));
			this.majJoueurs();
		} else if('chat'==réponse.type) {
			this.ligneChat(réponse.joueur,réponse.message);
		}
	};

	VueSalon.prototype.majJoueurs=function () {
		this.h1.lastChild.firstChild.textContent=this.app.salon.joueurs.length+' /3';
		if(this.app.salon.joueurs.length==3) {
			this.bouton.setAttribute('disabled','');
			// on tente d'afficher une notification
			if('webkitNotifications' in window
				&&window.webkitNotifications.checkPermission()===0) {
				var notification = window.webkitNotifications.createNotification(
					this.app.utilisateur.image1, 'Prêt à jouer',
					'La partie peut commencer, trois joueurs sont présents !');
				notification.show();
			}
		} else {
			this.bouton.setAttribute('disabled','disabled');
		}
	};

	VueSalon.prototype.ligneChat=function (joueur,message) {
			if(this.chat.firstChild)
				this.chat.appendChild(document.createElement('br'));
			if(joueur) {
				this.chat.appendChild(document.createElement('strong'));
				this.chat.lastChild.appendChild(document.createTextNode(joueur+' : '));
				this.chat.appendChild(document.createTextNode(message));
			} else {
				this.chat.appendChild(document.createTextNode(message));
			}
		this.chat.scrollTop=this.chat.scrollHeight;
	};

	// demande de permission pour les notifications
	VueSalon.prototype.notification=function (event) {
		// demande
		window.webkitNotifications.requestPermission();
		// suppression du bouton
		this.boutonNotification.style.display='none';
	};

	VueSalon.prototype.vider=function (event) {
		while(this.chat.firstChild)
			this.chat.removeChild(this.chat.firstChild);
	};

	VueSalon.prototype.uninit=function (app) {
		this.vider();
		this.bouton.setAttribute('disabled','disabled');
		Vue.prototype.uninit.bind(this)();
	};

	return VueSalon;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueMono',['Vue'], function (Vue) {

	// Vue pour jeu en solo
	function VueMono() { }

	// Hérite de Vue
	VueMono.prototype=new Vue();

	// Initialisation
	VueMono.prototype.init=function (app,name) {
		// Appel de la fonction parente
		Vue.prototype.init.bind(this)(app,name);
		// Enregistrement des commandes de la vue
		this.commande('retourner');
		this.commande('recommencer');
		// Acquisition d'une référence aux éléments d'affichage
		this.niveau=document.querySelector('div.vue.choisie ul.infos '
			+'li:first-child span').firstChild;
		this.temps=document.querySelector('div.vue.choisie ul.infos '
			+'li:nth-child(2) span').firstChild;
		this.essais=document.querySelector('div.vue.choisie ul.infos '
			+'li:nth-child(3) span').firstChild;
		// Chargement de la liste des univers de jeu
		var requete = new XMLHttpRequest();
		requete.open('GET','/univers/liste.json',false);
		requete.send(null);
		if(requete.status!=200)
			throw Error('Impossible de récupérer la liste des univers !');
		this.listeUnivers=JSON.parse(requete.responseText);
		// Mise en route
		this.recommencer();
	};

	// Recommence le jeu au début
	VueMono.prototype.recommencer=function () {
		// Mise à zéro du niveau courant
		this.niveau.textContent=1;
		// distribution des cartes
		this.distribuer();
	};

	// Commence un niveau
	VueMono.prototype.distribuer=function () {
		// Remise à 0 des essais
		this.essais.textContent=0;
		// Remise à 0 des paires trouvées
		this.paireTrouvées=0;
		// Récupération des cartes
		var pileCartes=Array.prototype.slice.call(
			document.querySelectorAll('div.vue.choisie div.cartes p'),0);
		// Chargement d'un fichier univers au hasard
		var requete = new XMLHttpRequest();
		var nomUnivers=this.listeUnivers[
			Math.floor(Math.random()*this.listeUnivers.length)];
		requete.open('GET','univers/'+nomUnivers+'.json',false);
		requete.send();
		if(requete.status!=200)
			throw Error('Impossible de récupérer l\'univers !');
		var univers=JSON.parse(requete.responseText);
		// Distribution des cartes
		this.cartes=[];
		// On itère pour chaque paire de cartes
		for(var i=(pileCartes.length/2)-1; i>=0; i--)	{
			// on retire deux cartes de la pile, au hasard
			var carte1=pileCartes.splice(Math.floor(Math.random()*pileCartes.length),1)[0];
			var carte2=pileCartes.splice(Math.floor(Math.random()*pileCartes.length),1)[0];
			// on sélectionne un type de carte dans l'univers actuel
			// on supprime ce type de l'univers pour éviter de le réutiliser
			carte1.typeCarte=carte2.typeCarte=univers.cartes.pop();
			// Si l'univers déclare un jeu d'image
			if(univers.image) {
				// on ajoute l'image en fond de la carte
				// et on modifie sa position sur l'axe des x
				// pour que la bonne image apparaisse
				carte1.firstChild.firstChild.style.backgroundImage='url(/images/'+univers.image+')';
				carte1.firstChild.firstChild.style.backgroundPosition='-'+(i*univers.largeurTuile)+'px center';
				carte2.firstChild.firstChild.style.backgroundImage='url(/images/'+univers.image+')';
				carte2.firstChild.firstChild.style.backgroundPosition='-'+(i*univers.largeurTuile)+'px center';
			}
			// sinon, on supprime les image de fond éventuelles
			else {
				carte1.firstChild.firstChild.style.backgroundImage='none';
				carte2.firstChild.firstChild.style.backgroundImage='none';
			}
			// on ajoute le texte du type de carte à la carte
			carte1.firstChild.firstChild.firstChild.firstChild.textContent=carte1.typeCarte.texte;
			carte2.firstChild.firstChild.firstChild.firstChild.textContent=carte2.typeCarte.texte;
			// on insère les cartes dans le jeu
			this.cartes.push(carte2.jumelle=carte1,carte1.jumelle=carte2);
		}
		// Remise à zéro des classes
		this.raz();
		// Remise à 0 des sons d'alerte
		this.alert1Sonnée=false;
		this.alert2Sonnée=false;
		// Remise à zéro du timer
		this.tempsMax=new Date(Date.now()+5000+(120000/parseInt(this.niveau.textContent,10)));
		// Mise en route du compte à rebours
		if(univers.image) {
			// on crée un nœud de type image
			var img=document.createElement('img');
			img.setAttribute('src','images/'+univers.image);
			// on démarre quand elle est chargée
			img.onload=this.majTemps.bind(this);
		}
		else
			this.majTemps();
	};

	// mise à jour du compte à rebours
	VueMono.prototype.majTemps=function () {
		// on supprime tout timeout préalable
		clearTimeout(this.timeoutTemps);
		// Différence entre tempsMax et moment actuel en secondes
		var tempsRestant=(this.tempsMax.getTime()-Date.now())/1000;
		// Si le temps restant est négatif, on le remet à 0
		if(0>=tempsRestant)
			tempsRestant=0;
		// Affichage dans l'élément adequat
		this.temps.textContent=Math.floor(tempsRestant/60)
			+'\''+(Math.floor(tempsRestant%60)<10?'0':'')+Math.floor(tempsRestant%60);
		// diffusion des alertes
		if(tempsRestant<20&&!this.alert1Sonnée) {
			this.app.sons.play('alerte1');
			this.temps.parentNode.classList.add('warn1');
			this.alert1Sonnée=true;
		}
		if(tempsRestant<10&&!this.alert2Sonnée) {
			this.app.sons.play('alerte2');
			this.temps.parentNode.classList.add('warn2');
			this.alert2Sonnée=true;
		}
		// Programmation du prochain affichage
		if(tempsRestant>0)
			this.timeoutTemps=setTimeout(this.majTemps.bind(this), 1000);
		// Plus de temps, le joueur a perdu
		else {
			this.timeoutTemps=null;
			this.app.sons.play('perdu');
			this.app.afficherMessage('Perdu !',3000,this.recommencer.bind(this));
		}
	};

	// Commande pour retourner une carte
	VueMono.prototype.retourner=function (event, params) {
		// on vérifie qu'on est pas entrain d'afficher un résultat
		if(this.timeoutResultat)
			return false;
		// récupération de la carte cliquée
		var carte=document.querySelector(
			'div.vue.choisie div.cartes p:nth-child('+params.carte+')');
		// on vérifie qu'elle n'est pas déjà trouvée ou sélectionnée
		if(carte.classList.contains('trouvee')
			||carte.classList.contains('choisie'))
			return false;
		// on la sélectionne
		carte.classList.add('face');
		carte.classList.add('choisie');
		// ajout de la couleur associée à la carte
		carte.firstChild.style.backgroundColor='#'+carte.typeCarte.couleur;
		// Si aucune carte choisie on garde une référence à celle-ci
		if(!this.carte1) {
			this.carte1=carte;
			this.app.sons.play('carte1');
		}
		// sinon, on vérifie si elle sont jumelles
		else {
			this.app.sons.play('carte2');
			this.carte2=carte;
			// si elles sont jumelle on appelle la méthode gagner
			// sinon on appelle la méthode perdre
			this.timeoutResultat=setTimeout(
				this[(this.carte1.jumelle==this.carte2?'gagner':'perdre')].bind(this),
				(this.carte1.jumelle==this.carte2?100:700));
		}
	};

	// action si la paire est pas bonne
	VueMono.prototype.perdre=function () {
		this.app.sons.play('pasbien');
		this.timeoutResultat=null;
		this.carte1.classList.toggle('face');
		this.carte1.classList.toggle('choisie');
		this.carte2.classList.toggle('face');
		this.carte2.classList.toggle('choisie');
		this.carte1=null;
		this.carte2=null;
		// on ajoute un essai infructueux
		this.essais.textContent=parseInt(this.essais.textContent,10)+1;
	};

	// action si la paire est bonne
	VueMono.prototype.gagner=function () {
		this.app.sons.play('bien');
		// on passe les deux carte en état trouvé
		this.timeoutResultat=null;
		this.carte1.classList.toggle('choisie');
		this.carte1.classList.toggle('trouvee');
		this.carte2.classList.toggle('choisie');
		this.carte2.classList.toggle('trouvee');
		// on réinitialise les cartes sélectionnées
		this.carte1=null;
		this.carte2=null;
		// on ajoute la paire trouvéee
		this.paireTrouvées++;
		// si toutes les paires sont trouvée
		if(this.paireTrouvées==this.cartes.length/2) {
			this.niveau.textContent=parseInt(this.niveau.textContent,10)+1;
			clearTimeout(this.timeoutTemps);
			this.timeoutTemps=null;
			// jouer le son d'applaudissements
			this.app.sons.play('applaudissements');
			// on félicite le joueur et on programme une nouvelle distribution
			this.app.afficherMessage('Bravo !',1500,this.distribuer.bind(this));
		}
	};

	// on remet la vue dans son état initial
	VueMono.prototype.raz=function () {
		this.carte1=null;
		this.carte2=null;
		if(this.timeoutResultat) {
			clearTimeout(this.timeoutResultat);
			this.timeoutResultat=null;
		}
		this.temps.parentNode.classList.remove('warn1');
		this.temps.parentNode.classList.remove('warn2');
		for(var i=this.cartes.length-1; i>=0; i--) {
			this.cartes[i].classList.remove('choisie');
			this.cartes[i].classList.remove('trouvee');
			this.cartes[i].classList.remove('face');
		}
	};

	// désinitialisation de la vue
	VueMono.prototype.uninit=function (app) {
		// suppression de tout callback de message
		this.app.messageCallback=null;
		// suppression du compte à rebours
		clearTimeout(this.timeoutTemps);
		// remise à zéro de la vue
		this.raz();
		// appel de la fonction parente
		Vue.prototype.uninit.bind(this)();
	};

	return VueMono;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueMaj',['Vue'], function (Vue) {

	// Vue pour mise à jour
	function VueMaj() { }

	// Hérite de Vue
	VueMaj.prototype=new Vue();

	// Initialisation
	VueMaj.prototype.init=function (app,name) {
		// Appel de la fonction parente
		Vue.prototype.init.bind(this)(app,name);
		// Enregistrement des commandes de la vue
		this.commande('mettreajour');
	};

	// Met à jour l'application
	VueMaj.prototype.mettreajour=function () {
		document.location.reload();
	};

	return VueMaj;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueConnexion',['Vue'], function (Vue) {

	function VueConnexion() { }

	VueConnexion.prototype=new Vue();

	VueConnexion.prototype.init=function (app,name) {
		Vue.prototype.init.bind(this)(app,name);
		this.commande('envoi');
	};

	VueConnexion.prototype.envoi=function (event, params) {
		this.app.utilisateur={nom:event.target[0].value};
		this.app.montrerVue('Salons');
	};

	VueConnexion.prototype.uninit=function (app) {
		Vue.prototype.uninit.bind(this)();
	};

	return VueConnexion;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

// AMD + global
(function(root,define){ define('VueNouveauSalon',['Vue'], function (Vue) {

	function VueNouveauSalon()	{ }

	VueNouveauSalon.prototype=new Vue();

	VueNouveauSalon.prototype.init=function (app,name) {
		// Appel de la fonction parente
		Vue.prototype.init.bind(this)(app,name);
		// référencement des zone dynamiques
		this.champs=document.querySelector('div.vue.choisie input[type="text"]');
		// Enregistrement des commandes de la vue
		this.commande('envoi');
		// ajout de l'écouteur de messages websockets
		this.app.connexionWS.onmessage=this.message.bind(this);
	};

	VueNouveauSalon.prototype.envoi=function (event, params) {
		this.app.connexionWS.send(JSON.stringify({'type':'nouveau','nom':this.champs.value}));
	};

	VueNouveauSalon.prototype.message=function (event) {
		var réponse=JSON.parse(event.data);
		if((!réponse.type))
			throw Error('Message innatendu de la part du serveur.');
		if('nouveau'==réponse.type) {
			this.app.montrerVue('Salons');
		}
	};

	VueNouveauSalon.prototype.uninit=function (app) {
		this.champs.value='';
		Vue.prototype.uninit.bind(this)();
	};

	return VueNouveauSalon;

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});

window.onload=function() {
	console.log('test');
}

if(!('bind' in Function.prototype)) {
	Function.prototype.bind = function(obj) {
		return function() {
			this.call(obj, arguments);
		}
	}
}

// pour analyse statique
function statique() {
	require(['VueSalons','VueProfil','VueOptions','VueSalons','VueSalon','VueMono','VueMaj','VueConnexion','VueNouveauSalon']);
}
// AMD + global
(function(root,define){ define('Application',['Vue','require',
		'libs/commandor/Commandor','libs/sounds/Sounds'],
	function (Vue,require,Commandor,Sounds) {

	// Objet application
	function Application(racine) {
		// Recherche d'une nouvelle version
		if(window.applicationCache) {
			window.applicationCache.addEventListener('updateready',function() {
				// on ne demande la mise à jour
				// que si le joueur n'est pas entrain de jouer
				if((!(root.VueMono||root.VueMulti))||this.vue instanceof root.VueMono
					||this.vue instanceof root.VueMulti)
					this.montrerVue('Maj');
			}.bind(this));
		}
		// stockage de l'élément HTML racine
		this.racine=racine;
		// création d'un gestionnaire de commande
		this.gestionnaire=new Commandor(racine);
		// création d'un gestionnaire de sons et ajout des sons
		this.sons=new Sounds('sounds',
			// joue le son de fond une fois chargé
			function() { this.sons.play('mountainman'); }.bind(this));
		this.sons.register('mountainman',['mid','mp3'],Infinity,0.2);
		this.sons.register('carte1',['ogg']);
		this.sons.register('carte2',['ogg']);
		this.sons.register('bien',['ogg','mp3']);
		this.sons.register('pasbien',['ogg']);
		this.sons.register('applaudissements',['ogg']);
		this.sons.register('alerte1',['ogg']);
		this.sons.register('alerte2',['ogg']);
		this.sons.register('perdu',['ogg']);
		this.sons.register('gagne',['ogg']);
		// Récupération de la préférence son coupé
		try {
			if(window.localStorage&&window.localStorage.sonCoupé)
				this.sons.mute(!!window.localStorage.sonCoupé);
		} catch(e) {}
		// Enregistrement de la commande de changement de vue
		this.gestionnaire.suscribe('changerVue',
			this.changerVue.bind(this));
		// stockage de l'élément pour afficher des messages
		this.message=document.querySelector('div.app div.message p');
		// on crée la variable contenant les essais websocket
		this.essaisWS=0;
		// on affiche la première vue
		this.montrerVue('Accueil');
	}

	// Gestion des messages
	Application.prototype.afficherMessage=function (texte,durée,callback) {
		if(this.messageTimeout)
			clearTimeout(this.messageTimeout);
		this.messageTimeout=setTimeout(this.cacherMessage.bind(this),durée||1500);
		this.messageCallback=callback;
		this.message.firstChild.textContent=texte;
		this.message.parentNode.classList.add('affiche');
	};

	Application.prototype.cacherMessage=function () {
		this.message.parentNode.classList.remove('affiche');
		this.messageTimeout=null;
		this.messageCallback&&this.messageCallback();
		this.messageCallback=null;
	};

	// Gestion des vues
	Application.prototype.changerVue=function (event,params) {
		this.montrerVue(params.vue);
	};

	Application.prototype.montrerVue=function (name) {
		// on désamorce la vue précédente
		if(this.vueAffichée)
			this.vueAffichée.uninit();
		// on lance la vue suivante
		// soit la vue a un constructeur dédié
		// soit on utilise le constructeur générique Vue
		console.log('Affichage vue '+name);
		// on teste d'abord le contexte global
		if(root.Vue) {
			if(root['Vue'+name])
				this.vueAffichée=new root['Vue'+name]();
			else
				this.vueAffichée=new Vue();
			this.vueAffichée.init(this,name);
		} else {
			// sinon, RequireJS
			require(['Vue'+name],function(VuePerso) {
				// succès
				this.vueAffichée=new VuePerso();
				this.vueAffichée.init(this,name);
			}.bind(this),function(){
				// échec
				this.vueAffichée=new Vue();
				this.vueAffichée.init(this,name);
			}.bind(this));
		}
	};

	// Gestion des websockets
	Application.prototype.connexion=function () {
		// on garde une référence au callback de message
		if(this.connexionWS&&this.connexionWS.onmessage)
			var retourMessage=this.connexionWS.onmessage;
		// si on a 3 essais de connexion infructueux
		if(this.essaisWS>2) {
			// on annule tout
			this.connexionWS.close();
			this.connexionWS=null;
			this.essaisWS=0;
			// on affiche la vue erreur réseau
			this.montrerVue('Reseau');
		}
console.log(WebSocket)
		// sinon, on retente une connexion
		this.essaisWS++;
		this.connexionWS = new WebSocket('ws://127.0.0.1:8124/ws');
		console.log(this.connexionWS);
		// on crée un timer pour abandonner la connexion
		// au bout de 5 secondes
		this.timeout=setTimeout(function() {
			if(this.essaisWS>0) {
				this.connexionWS.close();
				this.connexion();
			}
		}.bind(this),5000);
		// à l'ouverture de la connexion, on envoi les infos
		this.connexionWS.onopen = function () {
console.log('opened')
			if(retourMessage)
				this.connexionWS.onmessage=retourMessage;
			// on supprime le timeout
			if(this.timeout)
				clearTimeout(this.timeout);
			// on remet à zéro les essais
			this.essaisWS=0;
			// on envoi les infos utilisateur
			this.identification();
		}.bind(this);
		// on logge les erreurs pour débogage
		this.connexionWS.onerror = function (error) {
			console.log(error);
		}.bind(this);
		// on écoute toute fermeture
		this.connexionWS.onclose = function () {
			// on supprime le timeout
			if(this.timeout)
				clearTimeout(this.timeout);
			// On tente une reconnexion
			setTimeout(this.connexion.bind(this),2000);
		}.bind(this);
		// on écoute toute fermeture
		this.connexionWS.addEventListener('message', function (event) {
			console.log(event.data);
		});
	};

	// méthode d'envoi du profil
	Application.prototype.identification=function () {
		this.connexionWS.send(JSON.stringify({
			'type':'connexion',
			'nom':this.utilisateur.nom,
			'sexe':this.utilisateur.sexe,
			'image1':this.utilisateur.image1,
			'image2':this.utilisateur.image2,
			'sessid':(this.utilisateur.sessid||'')
		}));
	};

	// méthode de déconnexion
	Application.prototype.déconnexion=function () {
		// on supprime l'écouteur d'évènement
		this.connexionWS.onclose=null;
		// on ferme la connexion
		this.connexionWS.close();
		// on supprime toute référence à la connexion
		this.connexionWS=null;
	};

	// On lance l'application
	new Application(document.querySelector('div.app'));

});})(this,typeof define === 'function' && define.amd ? define : function (name, deps, factory) {
	this[name.substring(name.lastIndexOf('/')+1)]=factory.apply(this, deps.map(function(dep){
		return this[dep.substring(dep.lastIndexOf('/')+1)];
	}));
});
