// Self executing function with $ passed as a parameter
// to avoid conflicts with other libraries
(function( $ ){
     
   jQuery.fn.theSocialPlugin = function(options) {
      
      /********************************************************************************
   	/* PARAMETRES DU PLUGIN
   	/********************************************************************************/
      
      // Les options par dfaut du plugin  utiliser
      var optionsParDefaut = {
         srcFacebook: null,
         srcTwitter: null,
         nbFacebook: 5,   // Paramtre "limit" de Facebook
         nbTwitter: 5    // Paramtre "count" de Twitter
      };
      
      // On crase les options par dfaut par celles 
      // dfinies lors de l'appel du plugin
      var params = $.extend(optionsParDefaut, options);
   
   
      /********************************************************************************
   	/* MAIN BODY
   	/********************************************************************************/
   	
      if (params.srcFacebook != null || params.srcTwitter != null) {

         // this correspond au tableau d'objet jQuery
         // lors de l'appel du plugin theSocialPlugin
         
         // Message  afficher pendant le chargement
         this.html('Chargement des donnes...');

         // Initialisation du tableau qui contiendra les 
         // donnes (les posts) des flux Facebook et Twitter
         var lesPosts = [];

         // Sauvegarde la valeur de this pour 
         // la fonction .done()
         // Alternative: utiliser $.proxy()
         var $this = this;
         
         // Rcupre les donnes de Facebook et Twitter
         // Puis excution de la fonction "afficherResultats()"
         $.when(getFacebook(), getTwitter()).done(function() {
            afficherResultat($this);
         });
      }

      // Retourne "this" pour le chanage 
      // On peut ainsi faire: 
      // $('div').theSocialPlugin(...).css('color', 'red');
      return this;
      
      /********************************************************************************
   	/* FONCTIONS AUXILIAIRES - PROPRES AU PLUGIN
   	/********************************************************************************/
      
      function afficherResultat($elements) {

         var html = [];
         var i    = 0;
         
         // Titre
         var titre;
         if (params.srcFacebook != null && params.srcTwitter == null) {
            titre = 'Flux Facebook de ' + params.srcFacebook;
            
         } else if (params.srcTwitter != null && params.srcFacebook == null) {
            titre = 'Flux Twitter de ' + params.srcTwitter;
            
         } else {
            titre = 'Flux Facebook et Twitter de ' + params.srcFacebook;
         }
         
         html[i++] = '<h1>' + titre + '</h1>';
         
         
         // Contenu
         if (lesPosts.length == 0) {
            html[i++] = 'Aucune information  afficher';
            
         } else {
            
            // On trie les donnes par date de cration
            lesPosts.sort(function(el1, el2) {
               return el1.dtCreation < el2.dtCreation;
            });
            
            // Boucle sur chaque post  afficher
            $.each(lesPosts, function() {
               
               // Formattage de la date de cration
               var dt = $.datepicker.formatDate('dd/mm/yy', this.dtCreation);
               
               // URL vers la page facebook ou twitter
               var url;
               if (this.type == 'facebook') {
                  url = 'http://www.facebook.com/' + params.srcFacebook;
               } else {
                  url = 'http://twitter.com/' + params.srcTwitter;
               }
               
               // Lien vers l'article concern (peut tre vide)
               var lien = '';
               if (this.lien != '') {
                  lien = '<a href="' + this.lien + '">Plus d\'information</a><br />';
               }
               
               html[i++] = '<div>';
               html[i++] =    '<a href="' + url + '" class="' + this.type + '"></a>';
               html[i++] =    this.message + '<br />' + lien;
               html[i++] =    '<span>Le ' + dt + ' par ' + this.auteur + '</span>';
               html[i++] = '</div>';
            });
            
         }
         
         // Affiche le contenu dans les lments dom
         $elements
            .html( html.join('') )
            .addClass('theSocialPlugin');
         
      }
      
      
      // Rcupration du flux Facebook avec JSONP
      function getFacebook() {

         if (params.srcFacebook != null) {
            
            return $.ajax("https://graph.facebook.com/"
               + params.srcFacebook
               + "/posts?access_token=AAAAAAITEghMBAO8vvdlKUEi14lOTzK67OWcQp3kZA7YLSpO2hdP3GiUXvnAscTU3yEVSHwGlcGrYmH45vgqr2UvKvaKsfx290pSKwHOXkUDygW7En&limit=" + params.nbFacebook,
               {"dataType": "jsonp"}
            
            ).done(function(obj) {
               $.each(obj.data, function() {
                  // this = entre de obj.data
                  // ie: un post

                  // On parse la date de Facebook 
                  // avec jQuery UI
                  // Format ISO: 2011-03-12T09:47:57+0000
                  var dt = $.datepicker.parseDate('yy-mm-dd', 
                     this.created_time);

                  // Ajout du post Facebook au 
                  // tableau gnral "lesPosts"
                  lesPosts.push({
                     "type": "facebook",
                     "auteur": this.from.name,
                     "message": this.message,
                     "dtCreation": dt,
                     "lien": this.link
                  });
               });
            });
         }
         
         return null;
      }
      
      
      // Rcupration du flux Twitter avec JSONP
      function getTwitter() {
         
         if (params.srcTwitter != null) {
            
            return $.ajax("http://twitter.com/status/user_timeline/" 
               + params.srcTwitter + ".json"
               + "?count=" + params.nbTwitter,
               {"dataType": "jsonp"}

            ).done(function(posts) {

               $.each(posts, function() {
                  // this = entre de posts
                  // ie: un post
                  
                  // Regex pour dtecter le lien dans le message
                  var regex = new RegExp('^(.*)(https?://.*)$', 'i');
                  
                  // On enlve le lien du message
                  var message = this.text.replace(regex, '$1');
                  
                  // On stocke le lien pour le rutiliser plus tard
                  var lien    = this.text.replace(regex, '$2');
                  
                  // Exemple de Date provenant de Twitter
                  // Index: 0   1   2  3        4     5
                  // Date : Fri Mar 11 17:15:42 +0000 2011
                  var bits = this.created_at.split(' ');
                  var dt   = $.datepicker.parseDate('yyMdd', bits[5] + bits[1] + bits[2]);
                  
                  // Ajout du post Twitter au 
                  // tableau gnral "lesPosts"
                  lesPosts.push({
                     "type": "twitter",
                     "auteur": this.user.name,
                     "message": message,
                     "dtCreation": dt, 
                     "lien": lien
                  });
               });
            });
         }
         
         return null;
      }
   };
   
})( jQuery );
